//=============================================================================
//
//   File : BKConfig.cpp
//   Created on mar 06 ott 2009 22:40:48 by Iacopo Palazzi
//
//   This file is part of the BrisKola distribution
//   Copyright (C) 2009 Iacopo Palazzi <iacopo.palazzi@gmail.com>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================

#include "BKCard.h"
#include "BKConfig.h"
#include "BKController.h"
#include "BKDeck.h"
#include "BKGameBoard.h"
#include "BKGameView.h"
#include "BKGlobal.h"
#include "BKPlayer.h"
#include "BKMainWindow.h"

#include <QBuffer>
#include <QFSFileEngine>
#include <QHttp>
#include <QTextStream>
#include <QUrl>

#define _CONFIGURE_FILENAME QString(".briskola.conf")

// The one and only BKConfigobject
BKConfig * BKConfig::m_pInstance = NULL;

BKConfig::BKConfig()
	: QObject()
{
	m_pHTTP = new QHttp();
	connect(m_pHTTP, SIGNAL(done(bool)), this, SLOT(doCheckNewVersion(bool)));
	
	m_pReadVersion = new QBuffer();
	
	configure();
}

BKConfig::~BKConfig()
{
	save();
	
	if (m_pReadVersion) delete m_pReadVersion;
	if (m_pHTTP) delete m_pHTTP;
}

bool BKConfig::init()
{
	if(m_pInstance)return false;
	m_pInstance = new BKConfig();
	return true;
}

void BKConfig::done()
{
	if(!m_pInstance)return;
	delete m_pInstance;
	m_pInstance = NULL;
}

// ================================
// Start putting your stuff here...
// ================================

void BKConfig::setDeck(unsigned int p_uiDeck)
{
	char c = p_uiDeck + '0';
	m_pInstance->m_rImgMap[IMG_DECK] = _STR_DIR_IMG_DECKS + "/bkImgDeck_0" + QString(c) + ".png";
	
	/*
	BKGameBoard::setDeckImage();
	
	for(unsigned int i = 0; i < 40; ++i)
	{
		BKDeck::getCard(i)->card()->setDeck();
	}
	
	for(unsigned int i = 0; i < 3; ++i)
	{
		BKCard * temp = BKController::getCPU()->getCard(i);
		
		if(temp != NULL)
		{
			temp->setHidden(temp->isHidden());
		}
	}
	*/
	
	instance()->setInteger("Deck", p_uiDeck);
	
	emit(instance()->sgnDeckChanged());
}

int BKConfig::getDeck(int p_uiDefaultValue)
{
	return instance()->getInteger("Deck", p_uiDefaultValue);
}

void BKConfig::setBackground(unsigned int p_uiBack)
{
	m_pInstance->m_rImgMap[IMG_BACKGROUND] = _STR_DIR_IMG_BACKGROUND + "/bkImgBackground_0" + (p_uiBack + '0') + ".png";
	instance()->setInteger("Background", p_uiBack);
	emit (instance()->sgnBackgroundChanged());
}

int BKConfig::getBackground(int p_iDefaultValue = 0)
{
	return instance()->getInteger("Background", p_iDefaultValue);
}

void BKConfig::setKey(const char * p_strKey, const char * p_strKeyId, unsigned int p_uiValue)
{
	m_pInstance->setInteger(p_strKeyId, p_uiValue);

	switch(p_uiValue)
	{
		case 0:
			m_pInstance->setInteger(p_strKey, Qt::Key_A);
			break;
		case 1:
			m_pInstance->setInteger(p_strKey, Qt::Key_B);
			break;
		case 2:
			m_pInstance->setInteger(p_strKey, Qt::Key_C);
			break;
		case 3:
			m_pInstance->setInteger(p_strKey, Qt::Key_D);
			break;
		case 4:
			m_pInstance->setInteger(p_strKey, Qt::Key_E);
			break;
		case 5:
			m_pInstance->setInteger(p_strKey, Qt::Key_F);
			break;
		case 6:
			m_pInstance->setInteger(p_strKey, Qt::Key_G);
			break;
		case 7:
			m_pInstance->setInteger(p_strKey, Qt::Key_H);
			break;
		case 8:
			m_pInstance->setInteger(p_strKey, Qt::Key_I);
			break;
		case 9:
			m_pInstance->setInteger(p_strKey, Qt::Key_J);
			break;
		case 10:
			m_pInstance->setInteger(p_strKey, Qt::Key_K);
			break;
		case 11:
			m_pInstance->setInteger(p_strKey, Qt::Key_L);
			break;
		case 12:
			m_pInstance->setInteger(p_strKey, Qt::Key_M);
			break;
		case 13:
			m_pInstance->setInteger(p_strKey, Qt::Key_N);
			break;
		case 14:
			m_pInstance->setInteger(p_strKey, Qt::Key_O);
			break;
		case 15:
			m_pInstance->setInteger(p_strKey, Qt::Key_P);
			break;
		case 16:
			m_pInstance->setInteger(p_strKey, Qt::Key_Q);
			break;
		case 17:
			m_pInstance->setInteger(p_strKey, Qt::Key_R);
			break;
		case 18:
			m_pInstance->setInteger(p_strKey, Qt::Key_S);
			break;
		case 19:
			m_pInstance->setInteger(p_strKey, Qt::Key_T);
			break;
		case 20:
			m_pInstance->setInteger(p_strKey, Qt::Key_U);
			break;
		case 21:
			m_pInstance->setInteger(p_strKey, Qt::Key_V);
			break;
		case 22:
			m_pInstance->setInteger(p_strKey, Qt::Key_W);
			break;
		case 23:
			m_pInstance->setInteger(p_strKey, Qt::Key_X);
			break;
		case 24:
			m_pInstance->setInteger(p_strKey, Qt::Key_Y);
			break;
		case 25:
			m_pInstance->setInteger(p_strKey, Qt::Key_Z);
			break;
		case 26:
			m_pInstance->setInteger(p_strKey, Qt::Key_0);
			break;
		case 27:
			m_pInstance->setInteger(p_strKey, Qt::Key_1);
			break;
		case 28:
			m_pInstance->setInteger(p_strKey, Qt::Key_2);
			break;
		case 29:
			m_pInstance->setInteger(p_strKey, Qt::Key_3);
			break;
		case 30:
			m_pInstance->setInteger(p_strKey, Qt::Key_4);
			break;
		case 31:
			m_pInstance->setInteger(p_strKey, Qt::Key_5);
			break;
		case 32:
			m_pInstance->setInteger(p_strKey, Qt::Key_6);
			break;
		case 33:
			m_pInstance->setInteger(p_strKey, Qt::Key_7);
			break;
		case 34:
			m_pInstance->setInteger(p_strKey, Qt::Key_8);
			break;
		case 35:
			m_pInstance->setInteger(p_strKey, Qt::Key_9);
			break;
		case 36:
			m_pInstance->setInteger(p_strKey, Qt::Key_Space);
			break;
		case 37:
			m_pInstance->setInteger(p_strKey, Qt::Key_Up);
			break;
		case 38:
			m_pInstance->setInteger(p_strKey, Qt::Key_Down);
			break;
		case 39:
			m_pInstance->setInteger(p_strKey, Qt::Key_Right);
			break;
		case 40:
			m_pInstance->setInteger(p_strKey, Qt::Key_Left);
			break;
	}
}

void BKConfig::setTimeout(int p_iValue)
{
	instance()->m_iTimeOut = p_iValue;
	instance()->setInteger("Timeout", p_iValue);
}

void BKConfig::setVisiblePoints(bool p_bVisible)
{
	int iVisible = (p_bVisible ? 1 : 0);
	
	instance()->setInteger("PointsVisible", iVisible);
	
	emit(instance()->sgnVisiblePointsChanged(p_bVisible));
}

void BKConfig::setSquareAroundCards(bool p_bVisible)
{
	int iVisible = (p_bVisible ? 1 : 0);
	
	instance()->setInteger("SquareAroundCards", iVisible);
	
	emit(instance()->sgnSquareAroundCardsChanged(p_bVisible));
}

void BKConfig::setPlayerThatStarts(unsigned int p_uiPlayerThatStarts)
{
	instance()->setInteger("PlayerThatStarts", p_uiPlayerThatStarts);
}

void BKConfig::setDelayCPU(bool p_bDelay)
{
	int iDelay = (p_bDelay ? 1 : 0);
	
	instance()->setInteger("DelayCPU", iDelay);
}

bool BKConfig::getVisiblePoints(bool p_bDefaultValue = true)
{
	int iVisible = instance()->getInteger("PointsVisible", (p_bDefaultValue ? 1 : 0));
	
	return ((iVisible == 1) ? true : false);
}

bool BKConfig::getSquareAroundCards(bool p_bDefaultValue = true)
{
	int iVisible = instance()->getInteger("SquareAroundCards", (p_bDefaultValue ? 1 : 0));
	
	return ((iVisible == 1) ? true : false);
}

int BKConfig::getTimeout(int p_iDefaultValue = 1000)
{
	return instance()->getInteger("Timeout", p_iDefaultValue);
}

int BKConfig::getPlayerThatStarts(int p_iDefaultValue = 2)
{
	return instance()->getInteger("PlayerThatStarts", p_iDefaultValue);
}

int BKConfig::getKey(const char * p_strKey, int p_iDefaultValue)
{	
	return m_pInstance->getInteger(p_strKey, p_iDefaultValue);
}

int BKConfig::getKeyId(const char * p_strKeyId, int p_iDefaultValue)
{	
	return m_pInstance->getInteger(p_strKeyId, p_iDefaultValue);
}

bool BKConfig::getDelayCPU(bool p_bDefaultValue)
{
	int iDelay = instance()->getInteger("DelayCPU", (p_bDefaultValue ? 1 : 0));
	
	return ((iDelay == 1) ? true : false);
}

void BKConfig::configure()
{
	load();
	
	// Create Images MAP
	m_rImgMap[IMG_BACKGROUND] = _STR_DIR_IMG_BACKGROUND + "/bkImgBackground_0" + QString::number(getInteger("Background", 0)) + ".png";
	m_rImgMap[IMG_DECK]       = _STR_DIR_IMG_DECKS + "/bkImgDeck_0" + QString::number(getInteger("Deck", 1)) + ".png";
	m_rImgMap[IMG_MISC_TURN]  = _STR_DIR_IMG_MISC + "/bkSelector.png";
	m_rImgMap[IMG_MISC_WIN]   = _STR_DIR_IMG_MISC + "/bkImgWin.png";
	m_rImgMap[IMG_MISC_LOSE]  = _STR_DIR_IMG_MISC + "/bkImgLose.png";

	m_rImgMap[IMG_ICON_APP]  = _STR_DIR_IMG_ICONS + "/bkIcon.png";
	m_rImgMap[IMG_ICON_START]  = _STR_DIR_IMG_ICONS + "/bkIconGo.png";
	m_rImgMap[IMG_ICON_CLOSE]  = _STR_DIR_IMG_ICONS + "/bkIconClose.png";
	m_rImgMap[IMG_ICON_SETTINGS]  = _STR_DIR_IMG_ICONS + "/bkIconSettings.png";
	
	// Create Strings MAP
	m_rStringMap[TXT_APPNAME]        = _STR_TXT_APPNAME;
	m_rStringMap[TXT_VERSION]        = _STR_TXT_VERSION;
	m_rStringMap[TXT_VERSION_NAME]   = _STR_TXT_VERSION_NAME;
	
	m_rDisplayRect.setX(0);
	m_rDisplayRect.setY(0);
	m_rDisplayRect.setWidth(750);
	m_rDisplayRect.setHeight(500);
	
	m_bShowCPUCards = false;
	
	// Set the initial timeout
	m_iTimeOut = getInteger("Timeout", 1000);
}

void BKConfig::setInteger(const char * p_strLable, int p_iValue)
{
	set(p_strLable, QString::number(p_iValue));
}

int BKConfig::getInteger(const char * p_strLable, int p_iDefaultValue)
{
	return get(p_strLable, QString::number(p_iDefaultValue)).toInt();
}

void BKConfig::set(QString p_strLable, QString p_strValue)
{
	m_rConfs[p_strLable] = p_strValue;
}

QString BKConfig::get(QString p_strLable, QString p_strDefaultValue = "")
{
	if (!m_rConfs.contains(p_strLable))
		m_rConfs[p_strLable] = p_strDefaultValue;
	return m_rConfs[p_strLable];
}

void BKConfig::load()
{
	QString strUserPath = QFSFileEngine::homePath();
	QFile file(strUserPath + "/" + _CONFIGURE_FILENAME);
	
	m_rConfs.clear();
	
	file.open(QIODevice::ReadOnly);
	
	QTextStream stream( &file );
	QString line;
	const QString strSeparator("=");
	QStringList conf;
	int i = 1;
	while ( !stream.atEnd() ) {
		i++;
		
		line = stream.readLine(); // line of text excluding '\n'
		conf = line.split(strSeparator);
		
		if(conf.count() != 2)
			dbg("Warning %s [%d]: %s",
				_CONFIGURE_FILENAME.toStdString().c_str(),
				i,
				line.toStdString().c_str()
			);
		else
		{
			m_rConfs[conf[0].trimmed()] = conf[1].trimmed();
		}
	}
	
	file.close();
}

void BKConfig::save()
{
	QString strUserPath = QFSFileEngine::homePath();
	QFile file(strUserPath + "/" + _CONFIGURE_FILENAME);
	
	file.open(QIODevice::WriteOnly | QIODevice::Truncate);
	
	int line = 0;
	QString strLine;
	QMapIterator<QString, QString> i(m_rConfs);
	while (i.hasNext()) {
		i.next();
		line++;
		strLine = i.key() + " = " + i.value() + "\n";
		file.write(strLine.toAscii());
	}
	
	file.close();
}

void BKConfig::checkVersion()
{
	instance()->m_pHTTP->setHost("www.briskola.net");
	instance()->m_pHTTP->get(QString("/version.txt"), instance()->m_pReadVersion);
}

void BKConfig::doCheckNewVersion(bool error)
{
	m_pReadVersion->open(QIODevice::ReadOnly);
	
	if (m_pReadVersion->isOpen())
	{
		QString line;
		QTextStream stream( m_pReadVersion );
		while ( !stream.atEnd() ) {
			line = stream.readLine();
			// dbg(stream.readLine().toStdString().c_str()); // line of text excluding '\n;
		}
		if(!error)
		{
			if(line.startsWith("briskola_"))
			{
				if(line == (QString("briskola_") + _STR_TXT_VERSION))
					BKMainWindow::checkVersion(0);
				else
					BKMainWindow::checkVersion(1);	
			}
			else
			{
				BKMainWindow::checkVersion(2);
			}
		}
		else
		{
			BKMainWindow::checkVersion(2);
		}
	}
	else
	{
		dbg("m_pReadVersion: closed");
		return;
	}
	
	m_pReadVersion->close();
	
	dbg("doCheckNewVersion(%d)", error);
}
