#ifndef _BKCONFIG_H_
#define _BKCONFIG_H_
//=============================================================================
//
//   File : BKConfig.h
//   Created on mar 06 ott 2009 22:40:48 by Iacopo Palazzi
//
//   This file is part of the BrisKola distribution
//   Copyright (C) 2009 Iacopo Palazzi <iacopo.palazzi@gmail.com>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================

#include <QMap>
#include <QObject>
#include <QString>
#include <QRectF>

#include <QHttpResponseHeader>

#include "BKGlobal.h"

#define _CONFIG_KEY_		0
#define _CONFIG_KEY_BY_ID_	1
#define _CONFIG_TIMEOUT_	2

class QHttp;
class QBuffer;

class BKConfig : public QObject
{
	Q_OBJECT
protected:
	BKConfig();
	~BKConfig();

public:
	enum eImage {
		IMG_BACKGROUND,
		IMG_DECK,
		IMG_MISC_TURN,
		IMG_MISC_WIN,
		IMG_MISC_LOSE,
		IMG_ICON_APP,
		IMG_ICON_START,
		IMG_ICON_CLOSE,
		IMG_ICON_SETTINGS
	};
	
	enum eString {
		TXT_APPNAME,
		TXT_VERSION,
		TXT_VERSION_NAME
	};

private:
	enum eHTTPAction
	{
		HTTP_VERSION
	};

// ================================
// Put members declaration below...
// ================================
private:
	QRectF m_rDisplayRect;
	
	QMap<int, QString> m_rImgMap;
	QMap<int, QString> m_rStringMap;
	
	bool m_bShowCPUCards;
	
	int m_iTimeOut;
	
	QMap<QString, QString> m_rConfs;
	
	QHttp * m_pHTTP;
	
	QBuffer * m_pReadVersion;
	
public:

protected:


// ================================
// Put methods declaration below...
// ================================
private:
	void configure();
	
	void set(QString, QString);
	void set(const char * p_strLable, QString p_strValue) { set(QString(p_strLable), p_strValue); };
	QString get(QString, QString);
	QString get(const char * p_strLable, QString p_strDefaultValue) { return get(QString(p_strLable), p_strDefaultValue); };
	
	void setInteger(const char * p_strLable, int p_iValue);
	int getInteger(const char * p_strLable, int p_iDefaultValue);
	
	void load();
	void save();
	
public:
	static QString getImage(eImage p_eImage) { return m_pInstance->m_rImgMap.contains(p_eImage) ? m_pInstance->m_rImgMap[p_eImage] : QString(""); };
	static QString getString(eString p_eString) { return m_pInstance->m_rStringMap.contains(p_eString) ? m_pInstance->m_rStringMap[p_eString] : QString(""); };

	static QRectF getDisplayRect() { return m_pInstance->m_rDisplayRect; };

	static QString getCardImage(unsigned int p_uiSuit, unsigned int p_uiName) { return _STR_DIR_IMG_CARDS + "/" + QString::number(p_uiSuit) + QString::number(p_uiName) + ".png"; };

	static bool showCPUCards() { return m_pInstance->m_bShowCPUCards; };
	static void showCPUCards(bool p_bValue) { m_pInstance->m_bShowCPUCards = p_bValue; };
	
	static void setDeck(unsigned int p_uiDeck);
	static void setBackground(unsigned int p_uiBack);
	static void setKey(const char * p_strKey, const char * p_strKeyId, unsigned int p_uiValue);
	static void setTimeout(int p_iValue);
	static void setVisiblePoints(bool p_bVisible);
	static void setSquareAroundCards(bool p_bVisible);
	static void setPlayerThatStarts(unsigned int p_uiPlayerThatStarts);
	static void setDelayCPU(bool p_bDelay);

	static int getDeck(int p_uiDefaultValue);
	static int getBackground(int p_iDefaultValue);
	static int getTimeout(int p_iDefaultValue);
	static bool getVisiblePoints(bool p_bDefaultValue);
	static bool getSquareAroundCards(bool p_bDefaultValue);
	static int getPlayerThatStarts(int p_iDefaultValue);
	static int getKey(const char * p_strKey, int p_iDefaultValue);
	static int getKeyId(const char * p_strKeyId, int p_iDefaultValue);
	static bool getDelayCPU(bool p_bDefaultValue);
	
	static void checkVersion();

protected:


// Class follows singleton pattern
protected:
	static BKConfig * m_pInstance;
public:
	static BKConfig * instance() { return m_pInstance; };
	static bool init();
	static void done();

private slots:
	void doCheckNewVersion(bool error);

signals:
	void sgnBackgroundChanged();
	void sgnDeckChanged();
	void sgnVisiblePointsChanged(bool);
	void sgnSquareAroundCardsChanged(bool);
	void sgnNewVersion(int);
};

#endif //!_BKCONFIG_H_
