//=============================================================================
//
//   File : BKController.cpp
//   Created on mer 07 ott 2009 17:11:27 by Iacopo Palazzi
//
//   This file is part of the BrisKola distribution
//   Copyright (C) 2009 Iacopo Palazzi <iacopo.palazzi@gmail.com>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================

#include "BKController.h"

#include "BK.h"
#include "BKDeck.h"
#include "BKPlayerCPU.h"
#include "BKPlayerHUM.h"

// The one and only BKControllerobject
BKController * BKController::m_pInstance = NULL;

BKController::BKController()
: QObject()
{
	BKDeck::init();
	
	m_eState = STATE_IDLE;
	
	m_pPlayerCPU = new BKPlayerCPU(BKPlayer::PT_CPU);
	m_pPlayerHUM = new BKPlayerHUM(BKPlayer::PT_HUM);
	m_pMatch = new BKMatch(m_pPlayerCPU, m_pPlayerHUM);
	
	connect(m_pPlayerCPU, SIGNAL(sgnCard(BKCard *)), this, SLOT(sltChoosenCard(BKCard *)));
	connect(m_pPlayerHUM, SIGNAL(sgnCard(BKCard *)), this, SLOT(sltChoosenCard(BKCard *)));
}

BKController::~BKController()
{
	if (m_pMatch) delete m_pMatch;
	if (m_pPlayerCPU) delete m_pPlayerCPU;
	if (m_pPlayerHUM) delete m_pPlayerHUM;
	
	BKDeck::done();
}

bool BKController::init()
{
	if(m_pInstance)return false;
	m_pInstance = new BKController();
	return true;
}

void BKController::done()
{
	if(!m_pInstance)return;
	delete m_pInstance;
	m_pInstance = NULL;
}

// ================================
// Start putting your stuff here...
// ================================

void BKController::create()
{
}

void BKController::reset()
{
	((BKPlayerCPU *)m_pPlayerCPU)->stop();
	m_pMatch->reset();
}

void BKController::startGame()
{
	if (m_eState == STATE_PLAYING)
	{
		reset();
	}
	
	m_pMatch->start();
	
	m_eState = STATE_PLAYING;
}

void BKController::choosenCard(int p_uiCard)
{
	if (p_uiCard > 2 || p_uiCard < 0) return;
	
	if (match()->matchState() == BKMatch::MS_WAITING_HUM_CARD)
	{
		BKCard * card = getHUM()->getCard(p_uiCard);
		if (card == NULL) return;
		match()->choosenCard(card);
	}
}

//
// SLOTS
//

void BKController::sltChoosenCard(BKCard * p_pCard)
{
	//dbg(p_pCard->toString().toAscii());
	if (m_eState == STATE_PLAYING)
	{
		m_pMatch->choosenCard(p_pCard);
	}
}

void BKController::showCPUCards(bool p_bValue)
{
	if (m_pInstance)
	{
		BKCard * card;
		for(unsigned int i = 0; i < 3; i++)
		{
			if ((card = getCPU()->getCard(i)))
				card->setHidden(!p_bValue);
		}
		
		card = match()->get1STCard();
		if (card)
			card->setHidden(false);
			
		card = match()->get2STCard();
		if (card)
			card->setHidden(false);
		
	}
}
