//=============================================================================
//
//   File : BKMatch.cpp
//   Created on gio 08 ott 2009 14:43:13 by Iacopo Palazzi
//
//   This file is part of the BrisKola distribution
//   Copyright (C) 2009 Iacopo Palazzi <iacopo.palazzi@gmail.com>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================

#include "BKMatch.h"
#include "BKConfig.h"

#include <time.h>
#include <stdlib.h>

#include "BK.h"
#include "BKController.h"
#include "BKDeck.h"
#include "BKGameBoard.h"
#include "BKPlayerCPU.h"
#include "BKPlayerHUM.h"

#include <QString>

BKMatch::BKMatch(BKPlayer * p_pPlayerCPU, BKPlayer * p_pPlayerHUM)
{
	m_pShuffledDeck = new BKCard * [40];// (BKCard **) malloc (sizeof(BKCard *) * 40);
	m_pPlayers = new BKPlayer * [2]; //(BKPlayer **) malloc (sizeof(BKPlayer *) * 2);
	m_pPlayersOrder = new BKPlayer * [2]; //(BKPlayer **) malloc (sizeof(BKPlayer *) * 2);
	m_pPlayedCards = new BKCard * [2];
	m_uiPlayersPoints = new unsigned int [2];
	m_pNextHandTimeout = new QTimer();
	
	memset(m_pShuffledDeck, NULL, sizeof(BKCard *) * 40);
	m_pPlayers[0] = m_pPlayers[1] = NULL;
	m_pPlayersOrder[0] = m_pPlayersOrder[1] = NULL;
	m_pPlayedCards[0] = m_pPlayedCards[1] = NULL;
	m_uiPlayersPoints[0] = m_uiPlayersPoints[1] = 0;
	
	connect(m_pNextHandTimeout, SIGNAL(timeout()), this, SLOT(doKick()));
	
	m_pPlayers[BKPlayer::PT_CPU] = p_pPlayerCPU;
	m_pPlayers[BKPlayer::PT_HUM] = p_pPlayerHUM;
}

BKMatch::~BKMatch()
{
	if (m_pNextHandTimeout) delete m_pNextHandTimeout;
	if (m_uiPlayersPoints) delete [] m_uiPlayersPoints;
	if (m_pPlayedCards)	delete [] m_pPlayedCards;
	if (m_pPlayersOrder) delete [] m_pPlayersOrder;
	if (m_pPlayers) delete [] m_pPlayers;
	if (m_pShuffledDeck) delete [] m_pShuffledDeck;
}

void BKMatch::start()
{
	reset();
	shuffleDeck();
	giveInitialCards();
	doHand();
}

void BKMatch::reset()
{
	srand(time(0));
	
	m_uiHands = 0;
	
	BKGameBoard::reset();
	
	m_pPlayers[BKPlayer::PT_CPU]->reset();
	m_pPlayers[BKPlayer::PT_HUM]->reset();
	
	m_pPlayersOrder[0] = NULL;
	m_pPlayersOrder[1] = NULL;
	m_pBrisKola = NULL;
	
	m_uiPlayersPoints[0] = m_uiPlayersPoints[1] = 0;
	
	for(unsigned int i = 0; i < 40; i++)
		m_pShuffledDeck[i] = NULL;
}

void BKMatch::shuffleDeck()
{
	//dbg("[BKMatch]: Shuffling Deck");
	
	unsigned int i, r, arrTaken[40];
	
	memset(arrTaken, 0, sizeof(unsigned int) * 40);

	srand(m_uiSeed + time(0));

	for(i = 0; i < 40; i++)
	{
		while (arrTaken[r = (rand() % 40)] != 0);
		arrTaken[r] = 1;
		m_pShuffledDeck[i] = BKDeck::getCard(r);
	}

	m_uiSeed += time(0);

}

void BKMatch::giveInitialCards()
{
	unsigned int uiStarts = rand() % 2;
	int iPlayerThatStarts = BKConfig::getPlayerThatStarts(2); // 0: HUM starts; 1: CPU starts; 2: random choice
	
	if(iPlayerThatStarts == 2) // Random choice of first player
	{
		m_pPlayersOrder[HS_1ST_PLY] = m_pPlayers[(uiStarts + 0) % 2];
		m_pPlayersOrder[HS_2ST_PLY] = m_pPlayers[(uiStarts + 1) % 2];
	}
	else // 0: HUM starts; 1: CPU starts
	{
		m_pPlayersOrder[(iPlayerThatStarts + 0) % 2] = m_pPlayers[BKPlayer::PT_HUM];
		m_pPlayersOrder[(iPlayerThatStarts + 1) % 2] = m_pPlayers[BKPlayer::PT_CPU];
	}
	
	m_pPlayersOrder[HS_1ST_PLY]->assignCard(m_pShuffledDeck[0]); m_pShuffledDeck[0] = NULL;
	m_pPlayersOrder[HS_2ST_PLY]->assignCard(m_pShuffledDeck[1]); m_pShuffledDeck[1] = NULL;
	m_pPlayersOrder[HS_1ST_PLY]->assignCard(m_pShuffledDeck[2]); m_pShuffledDeck[2] = NULL;
	m_pPlayersOrder[HS_2ST_PLY]->assignCard(m_pShuffledDeck[3]); m_pShuffledDeck[3] = NULL;
	m_pPlayersOrder[HS_1ST_PLY]->assignCard(m_pShuffledDeck[4]); m_pShuffledDeck[4] = NULL;
	m_pPlayersOrder[HS_2ST_PLY]->assignCard(m_pShuffledDeck[5]); m_pShuffledDeck[5] = NULL;
	
	m_pBrisKola = m_pShuffledDeck[6]; m_pShuffledDeck[6] = NULL;
	
	m_pPlayersOrder[HS_1ST_PLY]->setSeenCard(m_pBrisKola);
	m_pPlayersOrder[HS_2ST_PLY]->setSeenCard(m_pBrisKola);
	
	unsigned int i;
	for(i = 6; i < 39; i++)
		m_pShuffledDeck[i] = m_pShuffledDeck[i + 1];
	m_pShuffledDeck[i] = m_pBrisKola;
	
	BKGameBoard::placeBrisKola(m_pBrisKola);
	BKGameBoard::showDeck(true);
	BKGameBoard::showGhostBrisKola(false);
	BKGameBoard::showHand();
	
	/*
	for(unsigned int j = 0; j < 2; j++)
	{
		eHandState hs = (j == 0) ? HS_1ST_PLY : HS_2ST_PLY;
		for(unsigned int i = 0; i < 3; i++)
		{
			if (m_pPlayersOrder[hs]->type() == BKPlayer::PT_CPU)
				BKGameBoard::placeCPUCard(i, m_pPlayersOrder[hs]->getCard(i));
			else
				BKGameBoard::placeHUMCard(i, m_pPlayersOrder[hs]->getCard(i));
		}
	}
	*/
	
}

void BKMatch::give2Cards()
{
	m_pPlayedCards[HS_1ST_PLY]->detach();
	m_pPlayedCards[HS_2ST_PLY]->detach();
	
	unsigned int i = 0;
	while(m_pShuffledDeck[i] == NULL) { i++; };
	
	if (m_uiHands < 17)
	{
		m_pPlayersOrder[HS_1ST_PLY]->assignCard(m_pShuffledDeck[i + 0]); m_pShuffledDeck[i + 0] = NULL;
		m_pPlayersOrder[HS_2ST_PLY]->assignCard(m_pShuffledDeck[i + 1]); m_pShuffledDeck[i + 1] = NULL;
	}
	else if (m_uiHands == 17)
	{
		m_pPlayersOrder[HS_1ST_PLY]->assignCard(m_pShuffledDeck[i]); m_pShuffledDeck[i] = NULL;
		m_pPlayersOrder[HS_2ST_PLY]->assignCard(m_pBrisKola);
		BKGameBoard::showDeck(false);
		BKGameBoard::showGhostBrisKola(true);
	}
}

void BKMatch::doHand()
{
	dbg("\n*** Starting HAND %2d ***", m_uiHands + 1);
	
	m_eHandState = HS_1ST_PLY;
	
	BKGameBoard::resetIcons();
	BKGameBoard::setTextHands(m_uiHands + 1);
	
	if (m_pPlayersOrder[HS_1ST_PLY]->type() == BKPlayer::PT_HUM)
	{
		doHUMPly();
	}
	else
	{
		doCPUPly();
	}
}

void BKMatch::doHUMPly()
{
	m_eMatchState = MS_WAITING_HUM_CARD;
	BKGameBoard::setHUMTurn();
}

void BKMatch::doCPUPly()
{
	m_eMatchState = MS_WAITING_CPU_CARD;
	((BKPlayerCPU *)m_pPlayersOrder[m_eHandState])->play();
	BKGameBoard::setCPUTurn();
}

void BKMatch::wakeHUMPly()
{
	m_pPlayedCards[m_eHandState] = m_pChoosenCard;
	
	BKGameBoard::placePlayedCard(m_eHandState, m_pPlayedCards[m_eHandState]);
	
	switch(m_eHandState)
	{
		case HS_1ST_PLY:
			m_eHandState = HS_2ST_PLY;
			doCPUPly();
		break;
		case HS_2ST_PLY:
			computeHand();
		break;
	}
}

void BKMatch::wakeCPUPly()
{
	m_pPlayedCards[m_eHandState] = m_pChoosenCard;
	
	BKGameBoard::placePlayedCard(m_eHandState, m_pPlayedCards[m_eHandState]);
	
	switch(m_eHandState)
	{
		case HS_1ST_PLY:
			m_eHandState = HS_2ST_PLY;
			doHUMPly();
		break;
		case HS_2ST_PLY:
			computeHand();
		break;
	}
}

void BKMatch::computeHand()
{
	m_uiHands++;
	
	BKPlayer * pWinner1, * pWinner2;
	eHandState eWinner;
	
	BKController::getHUM()->setSeenCard(m_pPlayedCards[HS_1ST_PLY]);
	BKController::getHUM()->setSeenCard(m_pPlayedCards[HS_2ST_PLY]);
	
	BKController::getCPU()->setSeenCard(m_pPlayedCards[HS_1ST_PLY]);
	BKController::getCPU()->setSeenCard(m_pPlayedCards[HS_2ST_PLY]);
	
	if (isBrisKola(m_pPlayedCards[HS_1ST_PLY]))
	{
		if (!isBrisKola(m_pPlayedCards[HS_2ST_PLY]))
		{
			eWinner = HS_1ST_PLY;
			pWinner1 = m_pPlayersOrder[HS_1ST_PLY];
			pWinner2 = m_pPlayersOrder[HS_2ST_PLY];
		}
		else
		{
			if (m_pPlayedCards[HS_1ST_PLY]->value() > m_pPlayedCards[HS_2ST_PLY]->value())
			{
				eWinner = HS_1ST_PLY;
				pWinner1 = m_pPlayersOrder[HS_1ST_PLY];
				pWinner2 = m_pPlayersOrder[HS_2ST_PLY];
			}
			else
			{
				eWinner = HS_2ST_PLY;
				pWinner1 = m_pPlayersOrder[HS_2ST_PLY];
				pWinner2 = m_pPlayersOrder[HS_1ST_PLY];
			}
		}
	}
	else
	{
		if (!isBrisKola(m_pPlayedCards[HS_2ST_PLY]))
		{
			if (m_pPlayedCards[HS_1ST_PLY]->suit() == m_pPlayedCards[HS_2ST_PLY]->suit())
			{
				if (m_pPlayedCards[HS_1ST_PLY]->value() > m_pPlayedCards[HS_2ST_PLY]->value())
				{
					eWinner = HS_1ST_PLY;
					pWinner1 = m_pPlayersOrder[HS_1ST_PLY];
					pWinner2 = m_pPlayersOrder[HS_2ST_PLY];
				}
				else
				{
					eWinner = HS_2ST_PLY;
					pWinner1 = m_pPlayersOrder[HS_2ST_PLY];
					pWinner2 = m_pPlayersOrder[HS_1ST_PLY];
				}
			}
			else
			{
				eWinner = HS_1ST_PLY;
				pWinner1 = m_pPlayersOrder[HS_1ST_PLY];
				pWinner2 = m_pPlayersOrder[HS_2ST_PLY];
			}
		}
		else
		{
			eWinner = HS_2ST_PLY;
			pWinner1 = m_pPlayersOrder[HS_2ST_PLY];
			pWinner2 = m_pPlayersOrder[HS_1ST_PLY];
		}
	}
	
	m_pPlayersOrder[HS_1ST_PLY] = pWinner1;
	m_pPlayersOrder[HS_2ST_PLY] = pWinner2;
	
	m_uiPlayersPoints[m_pPlayersOrder[HS_1ST_PLY]->type()] += m_pPlayedCards[HS_1ST_PLY]->points() + m_pPlayedCards[HS_2ST_PLY]->points();
	
	/*
	if (m_pPlayersOrder[HS_1ST_PLY]->type() == BKPlayer::PT_HUM)
	{
		BKGameBoard::setWin();
	}
	else
	{
		BKGameBoard::setLose();
	}
*/

	BKGameBoard::startBlinkingWinner(m_pPlayersOrder[HS_1ST_PLY]->type());
	
	// This is the winners points
	
	QString strCard = m_pPlayedCards[HS_1ST_PLY]->toString() + " >> " + m_pPlayedCards[HS_2ST_PLY]->toString();
	
	dbg(" * BrisKola: %s", m_pBrisKola->toString().toStdString().c_str());
	dbg(" * Cards: %s", strCard.toStdString().c_str());
	dbg(" * HUM: %d", m_uiPlayersPoints[BKPlayer::PT_HUM]);
	dbg(" * CPU: %d", m_uiPlayersPoints[BKPlayer::PT_CPU]);
	
	BKGameBoard::setTextCPUPoints(m_uiPlayersPoints[BKPlayer::PT_CPU]);
	BKGameBoard::setTextHUMPoints(m_uiPlayersPoints[BKPlayer::PT_HUM]);
	
	if (m_uiHands == 20)
	{
		m_eMatchState = MS_END;
		BKGameBoard::instance()->doSetVisiblePoints(true);
		BKGameBoard::instance()->doSetWinner(m_uiPlayersPoints[0]);
		dbg("\n*** Match Ended ***");
	}
	else
	{
		m_eMatchState = MS_WAITING_USER_KICK;
		//dbg("Waiting for User to kick ...");
		m_pNextHandTimeout->start(BKConfig::getTimeout(1000));
	}
}

void BKMatch::kick()
{
	m_pNextHandTimeout->stop();
	
	give2Cards();
	m_eMatchState = MS_STARTING_HAND;
	m_eHandState = HS_1ST_PLY;
	doHand();
}

bool BKMatch::isBrisKola(BKCard * p_pCard)
{
	return (p_pCard->suit() == m_pBrisKola->suit());
}

void BKMatch::choosenCard(BKCard * p_pCard)
{
	m_pChoosenCard = NULL;
	
	if (p_pCard == NULL) return;
	
	m_pChoosenCard = p_pCard;
	
	if (p_pCard->player()->type() == BKPlayer::PT_HUM && m_eMatchState == MS_WAITING_HUM_CARD)
	{
		wakeHUMPly();
	}
	else if (p_pCard->player()->type() == BKPlayer::PT_CPU && m_eMatchState == MS_WAITING_CPU_CARD)
	{
		wakeCPUPly();
	}
}

void BKMatch::doKick()
{
	kick();
}
