#ifndef _BKMatch_h_
#define _BKMatch_h_
//=============================================================================
//
//   File : BKMatch.h
//   Created on gio 08 ott 2009 14:43:13 by Iacopo Palazzi
//
//   This file is part of the BrisKola distribution
//   Copyright (C) 2009 Iacopo Palazzi <iacopo.palazzi@gmail.com>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================


///
/// \class BKMatch
/// \brief The BKMatch class
///
/// This class...
///

#include <QObject>
#include <QTimer>

#include "BKPlayer.h"

class BKCard;

class BKMatch : public QObject
{
	Q_OBJECT
public:

	///
	/// Creates an instance of BKMatch
	///
	BKMatch(BKPlayer * p_pPlayer1, BKPlayer * p_pPlayer2);

	///
	/// Destroys the instance of BKMatch
	/// and frees all the relevant resources
	///
	~BKMatch();

	enum eMatchState
	{
		MS_STARTING_HAND,
		MS_WAITING_HUM_CARD,
		MS_WAITING_CPU_CARD,
		MS_COMPUTING_HAND,
		MS_WAITING_USER_KICK,
		MS_END
	};
	
	enum eHandState
	{
		HS_1ST_PLY,
		HS_2ST_PLY
	};

// ================================
// Put members declaration below...
// ================================
private:
	BKCard ** m_pShuffledDeck;
	BKPlayer ** m_pPlayers;
	BKPlayer ** m_pPlayersOrder;
	
	BKCard ** m_pPlayedCards;
	BKCard * m_pChoosenCard;
	BKCard * m_pBrisKola;
	
	eMatchState m_eMatchState;
	eHandState m_eHandState;
	
	QTimer * m_pNextHandTimeout;

	unsigned int m_uiHands;
	unsigned int * m_uiPlayersPoints;

	unsigned int m_uiSeed;
	
public:

protected:


// ================================
// Put methods declaration below...
// ================================
private:
	void shuffleDeck();
	void giveInitialCards();
	void give2Cards();
	
	void doHand();
	void doHUMPly();
	void wakeHUMPly();
	void doCPUPly();
	void wakeCPUPly();
	void computeHand();

public:
	void reset();
	void start();
	void choosenCard(BKCard * p_pCard);
	void kick();
	bool isBrisKola(BKCard * p_pCard);
	
	BKCard * get1STCard() { return m_pPlayedCards[HS_1ST_PLY]; };
	BKCard * get2STCard() { return m_pPlayedCards[HS_2ST_PLY]; };
	BKCard * getCPUCard(unsigned int p_uiIdx) { return m_pPlayers[BKPlayer::PT_CPU]->getCard(p_uiIdx); };
	BKCard * getBrisKola() { return m_pBrisKola; };
	
	unsigned int getCPUPoints() { return m_uiPlayersPoints[BKPlayer::PT_CPU]; };
	unsigned int getHUMPoints() { return m_uiPlayersPoints[BKPlayer::PT_HUM]; };
	unsigned int getHand() { return m_uiHands; };
	
	eMatchState matchState() { return m_eMatchState; };
	eHandState handState() { return m_eHandState; };
	
protected:

public slots:
	void doKick();

}; // class BKMatch

#endif //!_BKMatch_h_
