//=============================================================================
//
//   File : BKPlayerAI.cpp
//   Created on gio 08 ott 2009 16:02:07 by Iacopo Palazzi
//
//   This file is part of the BrisKola distribution
//   Copyright (C) 2009 Iacopo Palazzi <iacopo.palazzi@gmail.com>
//   Copyright (C) 2009 Gianni Sammito <giannixsammito@gmail.com>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================

#include "BKPlayerAI.h"

#include "BK.h"
#include "BKController.h"
#include "BKDeck.h"
#include "BKMatch.h"
#include "BKPlayerCPU.h"
#include "BKConfig.h"

#define _GET_CPU_POINTS BKController::match()->getCPUPoints()
#define _GET_HUM_POINTS BKController::match()->getHUMPoints()
#define _GET_HAND BKController::match()->getHand()
#define _GET_CPU_CARD_(_X_) BKController::getCPU()->getCard(_X_)
#define _GET_HUM_CARD_(_X_) BKController::getHUM()->getCard(_X_)
#define _GET_MATCH_CARD_1 BKController::match()->get1STCard()
#define _BK_MAX_(_X_, _Y_) ((_X_ > _Y_) ? _X_ : _Y_)

BKPlayerAI::BKPlayerAI(BKPlayerCPU * p_pParent)
	: QThread()
{
	m_pParent = p_pParent;
}

BKPlayerAI::~BKPlayerAI()
{
}

void BKPlayerAI::run ()
{
	BKCard * m;
	
	switch(BKController::match()->handState())
	{
		case BKMatch::HS_1ST_PLY:
			m = imFirst();
		break;
		case BKMatch::HS_2ST_PLY:
			m = imSecond();
		break;
	}
	if(BKConfig::getDelayCPU(false))
		usleep(200000);
	m_pParent->choosenCard(m);
}

BKCard * BKPlayerAI::imSecond()
{
	BKCard * card, * brisKola;
	
	BKController::getCPU()->setSeenCard(_GET_MATCH_CARD_1);
	
	// ##################################################
	// Tests if CPU has less than 61 points
	// ##################################################
	if(_GET_CPU_POINTS <= 60)
	{
		// Tests if CPU points go over 59
		for(unsigned int i = 0; i < 3; i++)
		{
			if(!_GET_CPU_CARD_(i))
					continue;
			
			if(takesCard(_GET_CPU_CARD_(i), _GET_MATCH_CARD_1))
			{
				// Tests if with this take CPU points are equal or greater than 61 and at the same time
				// points taken in this hand are greater than 0
				if(_GET_CPU_POINTS + _GET_CPU_CARD_(i)->points() + _GET_MATCH_CARD_1->points() > 60 &&
						_GET_CPU_CARD_(i)->points() + _GET_MATCH_CARD_1->points() > 0)
				{
					return _GET_CPU_CARD_(i);
				}
			}
		}
	}
	
	if(_GET_HUM_POINTS <= 60 && _GET_HAND < 19)
	{
		// Tests if exists a CPU's choice that lets the human win
		if
		(
		   (_GET_CPU_CARD_(0) != NULL && HUMTakesAndWins(_GET_CPU_CARD_(0), _GET_MATCH_CARD_1)) ||
		   (_GET_CPU_CARD_(1) != NULL && HUMTakesAndWins(_GET_CPU_CARD_(1), _GET_MATCH_CARD_1)) ||
		   (_GET_CPU_CARD_(2) != NULL && HUMTakesAndWins(_GET_CPU_CARD_(2), _GET_MATCH_CARD_1))
		)
		{
			// True if HUM wins with all CPU's choices
			if(
				HUMTakesAndWins(_GET_CPU_CARD_(0), _GET_MATCH_CARD_1) &&
				HUMTakesAndWins(_GET_CPU_CARD_(1), _GET_MATCH_CARD_1) &&
				HUMTakesAndWins(_GET_CPU_CARD_(2), _GET_MATCH_CARD_1)
			)
			{
				; // Do nothing									
			}
			else
			{ // In this case exists at least a CPU's choice that doesn't let the human win
				unsigned int numOfCardsToNotPlay = 0;
				int cardsToNotPlay[2];
				cardsToNotPlay[0] = cardsToNotPlay[1] = -1;
				for(unsigned int i = 0; i < 3; ++i)
				{
					// If CPU plays Card-i human wins
					if(_GET_CPU_CARD_(i) != NULL && HUMTakesAndWins(_GET_CPU_CARD_(i), _GET_MATCH_CARD_1)) {
						cardsToNotPlay[numOfCardsToNotPlay] = i;
						++numOfCardsToNotPlay;
					}
				}
				// In this case CPU can play only one card
				if(_GET_HAND == 18 || numOfCardsToNotPlay == 2) {
					for(int i = 0; i < 3; ++i)
					{
						if(_GET_CPU_CARD_(i) != NULL && (i != cardsToNotPlay[0]) && (i != cardsToNotPlay[1]))
							return _GET_CPU_CARD_(i);
					}
				}
				else
				{ // In this case CPU can play two cards
					unsigned int cards[2], numCards = 0;
					for(int i = 0; i < 3; ++i) {
						if(i != cardsToNotPlay[0] && i != cardsToNotPlay[1] && _GET_CPU_CARD_(i) != NULL) {
							cards[numCards] = i;
							++numCards;
						}
					}
					
					// If card on table is briskola, CPU plays the least card
					if(BKController::match()->isBrisKola(_GET_MATCH_CARD_1)) {
						if(cardHasLessValue(_GET_CPU_CARD_(cards[0]), _GET_CPU_CARD_(cards[1])) == _GET_CPU_CARD_(cards[0]))
							return _GET_CPU_CARD_(cards[0]);
						else
							return _GET_CPU_CARD_(cards[1]);
						
					}
					else
					{ // In this case the card on table is not briskola
						// It's possible to take making points without using briskola
						if(
							canTakeWithPointsWithoutBrisKola(_GET_CPU_CARD_(cards[0]), _GET_MATCH_CARD_1) ||
							canTakeWithPointsWithoutBrisKola(_GET_CPU_CARD_(cards[1]), _GET_MATCH_CARD_1)
						)
						{	
							if(!canTakeWithPointsWithoutBrisKola(_GET_CPU_CARD_(cards[0]), _GET_MATCH_CARD_1))
								return _GET_CPU_CARD_(cards[1]);
							else if(!canTakeWithPointsWithoutBrisKola(_GET_CPU_CARD_(cards[1]), _GET_MATCH_CARD_1))
								return _GET_CPU_CARD_(cards[0]);
							else {
								if(_GET_CPU_CARD_(cards[0])->points() > _GET_CPU_CARD_(cards[1])->points())
									return _GET_CPU_CARD_(cards[0]);
								else
									return _GET_CPU_CARD_(cards[1]);
							}
						}
						else
						{ // In this case is not possible to take the card on table making points without briskola
							// Card on table is a leak or picture
							if(_GET_MATCH_CARD_1->isLeak() || _GET_MATCH_CARD_1->isPicture()) {
								if(cardHasLessValue(_GET_CPU_CARD_(cards[0]), _GET_CPU_CARD_(cards[1])) == _GET_CPU_CARD_(cards[0]))
									return _GET_CPU_CARD_(cards[0]);
								else
									return _GET_CPU_CARD_(cards[1]);
							}
							else // This is a Load
							{
								// It's not possible to take card on table
								if(
									!takesCard(_GET_CPU_CARD_(cards[0]), _GET_MATCH_CARD_1) &&
									!takesCard(_GET_CPU_CARD_(cards[1]), _GET_MATCH_CARD_1)
								)
								{
									if(cardHasLessValue(_GET_CPU_CARD_(cards[0]), _GET_CPU_CARD_(cards[1])) == _GET_CPU_CARD_(cards[0]))
										return _GET_CPU_CARD_(cards[0]);
									else
										return _GET_CPU_CARD_(cards[1]);
								
								}
								else
								{ // In this case CPU can take the card (a load) on table
									if(!takesCard(_GET_CPU_CARD_(cards[0]), _GET_MATCH_CARD_1))
										return _GET_CPU_CARD_(cards[1]);
									else if(!takesCard(_GET_CPU_CARD_(cards[1]), _GET_MATCH_CARD_1))
										return _GET_CPU_CARD_(cards[0]);
									
									else
									{ // Both cards take card on table
										if(cardHasLessValue(_GET_CPU_CARD_(cards[0]), _GET_CPU_CARD_(cards[1])) == _GET_CPU_CARD_(cards[0]))
											return _GET_CPU_CARD_(cards[0]);
										else
											return _GET_CPU_CARD_(cards[1]);
									}
								}
							}
						}
					}
				}
			}
		}
	}
	
	if(_GET_HAND == 16)
	{
		BKCard * CPUCards[3], * cardsNotSeen[3], * temp;
		unsigned int points1, points2, points3;
		
		CPUCards[0] = _GET_CPU_CARD_(0);
		CPUCards[1] = _GET_CPU_CARD_(1);
		CPUCards[2] = _GET_CPU_CARD_(2);
		
		for(unsigned int i = 0, j = 0; i < 40 && j < 3; ++i)
		{
			if(!BKController::getCPU()->seenCard(BKDeck::getCard(i)))
			{
				cardsNotSeen[j] = BKDeck::getCard(i);
				++j;
			}
		}
		
		brisKola = BKController::match()->getBrisKola();
		
		points1 = myMaxPointsHand17ImSecond(CPUCards, _GET_MATCH_CARD_1, brisKola, cardsNotSeen);
		
		temp = CPUCards[0];
		CPUCards[0] = CPUCards[1];
		CPUCards[1] = temp;
		
		points2 = myMaxPointsHand17ImSecond(CPUCards, _GET_MATCH_CARD_1, brisKola, cardsNotSeen);
		
		temp = CPUCards[0];
		CPUCards[0] = CPUCards[2];
		CPUCards[2] = temp;
		
		points3 = myMaxPointsHand17ImSecond(CPUCards, _GET_MATCH_CARD_1, brisKola, cardsNotSeen);
		
		// Looking for the card that maximizes my points
		if(points1 >= points2)
		{
			if(points1 == points2)
			{
				if(cardHasLessValue(_GET_CPU_CARD_(0), _GET_CPU_CARD_(1)) == _GET_CPU_CARD_(0))
				{
					if(points1 >= points3)
					{
						if(points1 == points3)
						{
							if(cardHasLessValue(_GET_CPU_CARD_(0), _GET_CPU_CARD_(2)) == _GET_CPU_CARD_(0))
								return _GET_CPU_CARD_(0);
							else
								return _GET_CPU_CARD_(2);
						}
						else
						{
							return _GET_CPU_CARD_(0);
						}
					}
					else
						return _GET_CPU_CARD_(2);
				}
				else
				{
					if(points2 >= points3)
					{
						if(points2 == points3)
						{
							if(cardHasLessValue(_GET_CPU_CARD_(1), _GET_CPU_CARD_(2)) == _GET_CPU_CARD_(1))
								return _GET_CPU_CARD_(1);
							else
								return _GET_CPU_CARD_(2);
						}
						else
						{
							return _GET_CPU_CARD_(1);
						}
					}
					else
						return _GET_CPU_CARD_(2);
				}
			}
			else
			{
				if(points1 >= points3)
				{
					if(points1 == points3)
					{
						if(cardHasLessValue(_GET_CPU_CARD_(0), _GET_CPU_CARD_(2)) == _GET_CPU_CARD_(0))
							return _GET_CPU_CARD_(0);
						else
							return _GET_CPU_CARD_(2);
					}
					else
					{
						return _GET_CPU_CARD_(0);
					}	
				}
				else
					return _GET_CPU_CARD_(2);
			}
		}
		else
		{
			if(points2 >= points3)
			{
				if(points2 == points3)
				{
					if(cardHasLessValue(_GET_CPU_CARD_(1), _GET_CPU_CARD_(2)) == _GET_CPU_CARD_(1))
						return _GET_CPU_CARD_(1);
					else
						return _GET_CPU_CARD_(2);
				}
				else
				{
					return _GET_CPU_CARD_(1);
				}
			}
			else
				return _GET_CPU_CARD_(2);
		}
	}
	else if(_GET_HAND == 17)
	{
		BKCard * CPUCards[3], * HUMCards[3];
		unsigned int points1, points2, points3;
		
		HUMCards[0] = _GET_MATCH_CARD_1;
		
		for(unsigned int i = 0, j = 1; i < 3; ++i)
		{
			if(_GET_HUM_CARD_(i) != _GET_MATCH_CARD_1)
			{
				HUMCards[j] = _GET_HUM_CARD_(i);
				++j;
			}
		}
		
		// In this case CPU plays _GET_CPU_CARD_(0)
		CPUCards[0] = _GET_CPU_CARD_(0);
		CPUCards[1] = _GET_CPU_CARD_(1);
		CPUCards[2] = _GET_CPU_CARD_(2);
		points1 = myMaxPointsHand18ImSecond(CPUCards, HUMCards);
		
		// In this case CPU plays _GET_CPU_CARD_(1)
		CPUCards[0] = _GET_CPU_CARD_(1);
		CPUCards[1] = _GET_CPU_CARD_(0);
		CPUCards[2] = _GET_CPU_CARD_(2);
		points2 = myMaxPointsHand18ImSecond(CPUCards, HUMCards);
		
		// In this case CPU plays _GET_CPU_CARD_(2)
		CPUCards[0] = _GET_CPU_CARD_(2);
		CPUCards[1] = _GET_CPU_CARD_(1);
		CPUCards[2] = _GET_CPU_CARD_(0);
		points3 = myMaxPointsHand18ImSecond(CPUCards, HUMCards);
		
		if(points1 > points2)
		{
			if(points1 > points3)
				return _GET_CPU_CARD_(0);
			else
				return _GET_CPU_CARD_(2);
		}
		else
		{
			if(points2 > points3)
				return _GET_CPU_CARD_(1);
			else
				return _GET_CPU_CARD_(2);
		}
	}
	else if(_GET_HAND == 18)
	{
		BKCard * cardsCPU[2], * cardHUM, * cardOnTable;
	
		for(unsigned int i = 0, j = 0; i < 3; ++i)
		{ // Copy CPU cards in array cardsCPU
			if(_GET_CPU_CARD_(i) != NULL)
			{
				cardsCPU[j] = _GET_CPU_CARD_(i);
				++j;
			}
		}
	
		cardOnTable = _GET_MATCH_CARD_1;
	
		for(unsigned int i = 0; i < 3; ++i)
		{ // Get the card of Human not on table
			cardHUM = _GET_HUM_CARD_(i);
			if(cardHUM != NULL && cardHUM != cardOnTable)
				break;
		}
		
		if(myMaxPointsHand19ImSecond(cardsCPU[0], cardsCPU[1], cardOnTable, cardHUM) >
				myMaxPointsHand19ImSecond(cardsCPU[1], cardsCPU[0], cardOnTable, cardHUM))
			return cardsCPU[0];
		else
			return cardsCPU[1];
	}
	
	unsigned int pointsTaken = 0, res;
	int canTakeWithNoPoints = -1;
	
	// Tests if CPU can take doing points withouth using briskola
	for(unsigned int i = 0; i < 3; i++)
	{
		if(_GET_CPU_CARD_(i) == NULL)
			continue;
		
		if(
			!BKController::match()->isBrisKola(_GET_CPU_CARD_(i)) &&
			takesCard(_GET_CPU_CARD_(i), _GET_MATCH_CARD_1) /*&&
			(isPicture(_GET_CPU_CARD_(i)) || isLoad(_GET_CPU_CARD_(i)))*/
		)
		{
			// The condition is true if is possible to take the leak on table with another leak (both non-briskola)
			if(_GET_CPU_CARD_(i)->points() == 0)
				canTakeWithNoPoints = i;
				
			if(pointsTaken < _GET_CPU_CARD_(i)->points())
			{ // Looking for the card (not briskola) that take the card on table maximizing points
				pointsTaken = _GET_CPU_CARD_(i)->points();
				res = i;
			}
		}
	}
	
	if(pointsTaken) // If pointsTake > 0 it's possible to take card on table making points without using briskola
		return _GET_CPU_CARD_(res);
	
	if(!BKController::match()->isBrisKola(_GET_MATCH_CARD_1))
	{ // In this case there is a non-briskola on the table
		pointsTaken = 10;
		
		if(_GET_MATCH_CARD_1->isLoad())
		{ // Card on table is a Load and we're looking for the lowest briskola
			for(unsigned int i = 0; i < 3; i++)
			{
				if(_GET_CPU_CARD_(i) == NULL)
					continue;
				
				if(BKController::match()->isBrisKola(_GET_CPU_CARD_(i)) && pointsTaken > _GET_CPU_CARD_(i)->value())
				{ // Looking for the lowest briskola
					res = i;
					pointsTaken = _GET_CPU_CARD_(i)->value();
				}
			}
			
			if(pointsTaken < 10)
			{ // If pointsTaken < 10 CPU has at least a briskola
				return _GET_CPU_CARD_(res);
			}
			
			// At this point CPU can't take the load, so choose the card with lowest points
			pointsTaken = 12;
			
			for(unsigned int i = 0; i < 3; i++)
			{
				if(_GET_CPU_CARD_(i) == NULL)
					continue;
					
				if(_GET_CPU_CARD_(i)->points() < pointsTaken)
				{ // Looking for the card with lowest points
					pointsTaken = _GET_CPU_CARD_(i)->points();
					res = i;
				}
			}
			
			return _GET_CPU_CARD_(res);
		}
		else
		{ // Card on table is a Leak or a Load
			if(_GET_MATCH_CARD_1->isLeak())
			{ // Handling a LEAK
				
				pointsTaken = 10;
				
				for(unsigned int i = 0; i < 3; i++)
				{
					if(_GET_CPU_CARD_(i) == NULL)
						continue;
					
					if(!takesCard(_GET_CPU_CARD_(i), _GET_MATCH_CARD_1) && pointsTaken > _GET_CPU_CARD_(i)->value())
					{ // Looking for the card that doesn't take leak on table and minimizes points given
						res = i;
						pointsTaken = _GET_CPU_CARD_(i)->value();
					}
				}
				
				// If the card that should take the human has Points and CPU can take with no points, take with no points
				if(pointsTaken >= 5 && canTakeWithNoPoints >= 0)
					return _GET_CPU_CARD_(canTakeWithNoPoints);
				
				if(pointsTaken >= 0 && pointsTaken <= 7)
				{ // Looking for a card (not a load and not briskola) that human takes
					return _GET_CPU_CARD_(res);
				}
				else
				{ // In this case cards that human takes are all loads and the other ones are briskola
					pointsTaken = 10;
					
					for(unsigned int i = 0; i < 3; i++)
					{
						if(_GET_CPU_CARD_(i) == NULL)
							continue;
						
						if(
							BKController::match()->isBrisKola(_GET_CPU_CARD_(i)) &&
							pointsTaken > _GET_CPU_CARD_(i)->value()
						)
						{ // Looking for the briskola with minimum value
							res = i;
							pointsTaken = _GET_CPU_CARD_(i)->value();
						}
					}
					
					if(pointsTaken < 10)
					{ // Looking for the lowest briskola
						return _GET_CPU_CARD_(res);
					}
					
					else
					{ // CPU hasn't any briskola, so looks for the lowest load
						pointsTaken = 10;
						
						for(unsigned int i = 0; i < 3; i++)
						{
							if(_GET_CPU_CARD_(i) == NULL)
								continue;
							
							if(pointsTaken > _GET_CPU_CARD_(i)->value())
							{ // Looking for the card with lowest value
								res = i;
								pointsTaken = _GET_CPU_CARD_(i)->value();
							}
						}
						
						return _GET_CPU_CARD_(res);
					}
				}
			}
			else
			{ // Handling a PICTURE
				if(_GET_HUM_POINTS > _GET_CPU_POINTS + 12 && _GET_HUM_POINTS >= 30)
				{ // HUM has more that 30 points and at least 13 points more than CPU
					// In this case CPU try to take with a briskola, if it's not a load
					pointsTaken = 10;
					
					for(unsigned int i = 0; i < 3; i++)
					{
						if(_GET_CPU_CARD_(i) == NULL)
							continue;
						
						if(takesCard(_GET_CPU_CARD_(i), _GET_MATCH_CARD_1) && pointsTaken > _GET_CPU_CARD_(i)->value())
						{ // Looking for the b wiriskolath minimum value that takes picture on table
							res = i;
							pointsTaken = _GET_CPU_CARD_(i)->value();
						}
					}
					
					// If the briskola is not a load
					if(pointsTaken >= 0 && pointsTaken <= 7)
					{
						return _GET_CPU_CARD_(res);
					}
				}
				
				// HUM has not 12 points more than CPU, so CPU try to not take the card on table
				res = tryToNotTakeSoft();
				return _GET_CPU_CARD_(res);
			}
		}
	}
	else
	{ // Card on table is briskola
		res = tryToNotTakeHard();
		return _GET_CPU_CARD_(res);
	}

	
	unsigned int i = 0;
	while((card = BKController::getCPU()->getCard(i)) == NULL && i < 3) { i++; }
	
	return card;
}

BKCard * BKPlayerAI::imFirst()
{
	BKCard * card0, * card1, * card2, * res, * brisKola;
	
	brisKola = BKController::match()->getBrisKola();
	
	if(_GET_HAND == 16)
	{
		BKCard * cardsNotSeenByHUM[4], * cardsNotSeenByCPU[4], * myCards[3], * temp;
		unsigned int points1, points2, points3;
		
		myCards[0] = _GET_CPU_CARD_(0);
		myCards[1] = _GET_CPU_CARD_(1);
		myCards[2] = _GET_CPU_CARD_(2);
		
		for(unsigned int i = 0, j_H = 0, j_C = 0; i < 40; ++i)
		{
			if(!BKController::getHUM()->seenCard(BKDeck::getCard(i)))
			{
				cardsNotSeenByHUM[j_H] = BKDeck::getCard(i);
				++j_H;
			}
			if(!BKController::getCPU()->seenCard(BKDeck::getCard(i)))
			{
				cardsNotSeenByCPU[j_C] = BKDeck::getCard(i);
				++j_C;
			}
		}
		
		// Points if I play myCards[0]
		points1 = maxOpponentPointsHand17ImFirst(myCards, cardsNotSeenByCPU, cardsNotSeenByHUM, brisKola);
		
		// Points if I play myCards[1]
		temp = myCards[0];
		myCards[0] = myCards[1];
		myCards[1] = temp;
		points2 = maxOpponentPointsHand17ImFirst(myCards, cardsNotSeenByCPU, cardsNotSeenByHUM, brisKola);
		
		// Points if I play myCards[2]
		temp = myCards[0];
		myCards[0] = myCards[2];
		myCards[2] = temp;
		points3 = maxOpponentPointsHand17ImFirst(myCards, cardsNotSeenByCPU, cardsNotSeenByHUM, brisKola);
		
		// Looking for the card that minimizes HUM's points
		if(points1 <= points2)
		{
			if(points1 == points2)
			{
				if(cardHasLessValue(_GET_CPU_CARD_(0), _GET_CPU_CARD_(1)) == _GET_CPU_CARD_(0))
				{
					if(points1 <= points3)
					{
						if(points1 == points3)
						{
							if(cardHasLessValue(_GET_CPU_CARD_(0), _GET_CPU_CARD_(2)) == _GET_CPU_CARD_(0))
								return _GET_CPU_CARD_(0);
							else
								return _GET_CPU_CARD_(2);
						}
						else
						{
							return _GET_CPU_CARD_(0);
						}
					}
					else
						return _GET_CPU_CARD_(2);
				}
				else
				{
					if(points2 <= points3)
					{
						if(points2 == points3)
						{
							if(cardHasLessValue(_GET_CPU_CARD_(1), _GET_CPU_CARD_(2)) == _GET_CPU_CARD_(1))
								return _GET_CPU_CARD_(1);
							else
								return _GET_CPU_CARD_(2);
						}
						else
						{
							return _GET_CPU_CARD_(1);
						}
					}
					else
						return _GET_CPU_CARD_(2);
				}
			}
			else
			{
				if(points1 <= points3)
				{
					if(points1 == points3)
					{
						if(cardHasLessValue(_GET_CPU_CARD_(0), _GET_CPU_CARD_(2)) == _GET_CPU_CARD_(0))
							return _GET_CPU_CARD_(0);
						else
							return _GET_CPU_CARD_(2);
					}
					else
					{
						return _GET_CPU_CARD_(0);
					}	
				}
				else
					return _GET_CPU_CARD_(2);
			}
		}
		else
		{
			if(points2 <= points3)
			{
				if(points2 == points3)
				{
					if(cardHasLessValue(_GET_CPU_CARD_(1), _GET_CPU_CARD_(2)) == _GET_CPU_CARD_(1))
						return _GET_CPU_CARD_(1);
					else
						return _GET_CPU_CARD_(2);
				}
				else
				{
					return _GET_CPU_CARD_(1);
				}
			}
			else
				return _GET_CPU_CARD_(2);
		}
	}
	else if(_GET_HAND == 17)
	{
		BKCard * CPUCards[3], * HUMCards[3];
		BKCard * temp;
		unsigned int points1, points2, points3;
		
		for(unsigned int i = 0; i < 3; ++i)
		{
			CPUCards[i] = _GET_CPU_CARD_(i);
			HUMCards[i] = _GET_HUM_CARD_(i);
		}
		
		// In this case CPU plays _GET_CPU_CARD_(0)
		points1 = maxOpponentPointsHand18ImFirst(CPUCards, HUMCards);
		
		// In this case CPU plays _GET_CPU_CARD_(1)
		temp = CPUCards[0];
		CPUCards[0] = CPUCards[1];
		CPUCards[1] = temp;
		points2 = maxOpponentPointsHand18ImFirst(CPUCards, HUMCards);
		
		// In this case CPU plays _GET_CPU_CARD_(2)
		temp = CPUCards[0];
		CPUCards[0] = CPUCards[2];
		CPUCards[2] = temp;
		points3 = maxOpponentPointsHand18ImFirst(CPUCards, HUMCards);
		
		if(points1 < points2)
		{
			if(points1 < points3)
				return _GET_CPU_CARD_(0);
			else
				return _GET_CPU_CARD_(2);
		}
		else
		{
			if(points2 < points3)
				return _GET_CPU_CARD_(1);
			else
				return _GET_CPU_CARD_(2);
		}
		
	}
	else if(_GET_HAND == 18)
	{
		BKCard * CPUCard[2], * HUMCard[2];
		
		for(unsigned int i = 0, j = 0; i < 3; ++i)
		{
			if(_GET_CPU_CARD_(i) == NULL)
				continue;
			CPUCard[j] = _GET_CPU_CARD_(i);
			++j;
		}
		
		for(unsigned int i = 0, j = 0; i < 3; ++i)
		{
			if(_GET_HUM_CARD_(i) == NULL)
				continue;
			HUMCard[j] = _GET_HUM_CARD_(i);
			++j;
		}
		
		if(maxOpponentPointsHand19ImFirst(CPUCard[0], CPUCard[1], HUMCard[0], HUMCard[1]) >
				maxOpponentPointsHand19ImFirst(CPUCard[1], CPUCard[0], HUMCard[0], HUMCard[1]))
			return CPUCard[1];
		else
			return CPUCard[0];
	}
	
	card0 = BKController::getCPU()->getCard(0);
	card1 = BKController::getCPU()->getCard(1);
	card2 = BKController::getCPU()->getCard(2);
	
	res = cardHasLessValue(card0, card1);
	res = cardHasLessValue(res, card2);
	
	return res;
}

bool BKPlayerAI::HUMTakesAndWins(BKCard * p_pCardThatCPUCanPlay, BKCard * p_pCardOnTable)
{
	if(!p_pCardThatCPUCanPlay)
		return TRUE;
	
	// True if HUM wins the hand and the match
	if(!takesCard(p_pCardThatCPUCanPlay, p_pCardOnTable) && ((_GET_HUM_POINTS + p_pCardThatCPUCanPlay->points() + p_pCardOnTable->points()) > 60))
		return TRUE;
	else
		return FALSE;
}

// Return TRUE if Card p_pCardToPlay takes Card p_pCardOnTable, FALSE otherwise
bool BKPlayerAI::takesCard(BKCard * p_pCardToPlay, BKCard * p_pCardOnTable)
{
	unsigned int uiSuitCardToPlay, uiSuitCardOnTable;
	unsigned int uiValueCardToPlay, uiValueCardOnTable;
	
	uiSuitCardToPlay = p_pCardToPlay->suit();
	uiSuitCardOnTable = p_pCardOnTable->suit();
	
	uiValueCardToPlay = p_pCardToPlay->value();
	uiValueCardOnTable = p_pCardOnTable->value();
	
	if(uiSuitCardToPlay == uiSuitCardOnTable)
	{
		if(uiValueCardToPlay > uiValueCardOnTable)
			return TRUE;
		else
			return FALSE;
	}
	else if(BKController::match()->isBrisKola(p_pCardToPlay) || BKController::match()->isBrisKola(p_pCardOnTable))
	{
		return BKController::match()->isBrisKola(p_pCardToPlay);
	}
	else
		return FALSE;
}

BKCard * BKPlayerAI::cardHasLessValue(BKCard * p_pCard1, BKCard * p_pCard2)
{
	BKCard * pBrisKola = BKController::match()->getBrisKola();
	
	if (!p_pCard1 && !p_pCard2)
		return NULL;
		
	if (!p_pCard1)
		return p_pCard2;
		
	if (!p_pCard2)
		return p_pCard1;
	
	if (
		p_pCard1->points() + ((p_pCard1->suit() == pBrisKola->suit()) ? 5 : 0)
		<=
		p_pCard2->points() + ((p_pCard2->suit() == pBrisKola->suit()) ? 5 : 0)
	   )
	{
		if(
			(p_pCard1->points() + ((p_pCard1->suit() == pBrisKola->suit()) ? 5 : 0)) == (p_pCard2->points() + ((p_pCard2->suit() == pBrisKola->suit()) ? 5 : 0))
		) {
			if(pointsThatCanTake(p_pCard1) < pointsThatCanTake(p_pCard2)) {
				return p_pCard1;
			}
			else {
				return p_pCard2;
			}
		}
		else {
			return p_pCard1;
		}
	}
	else {
		return p_pCard2;
	}
}

unsigned int BKPlayerAI::pointsThatCanTake(BKCard * p_pCard)
{
	unsigned int points = 0, pointsOfCardI;
	
	pointsOfCardI = p_pCard->points();
	
	// If card is not greater than 3
	if(pointsOfCardI <= 10)
		points = points + 11 * (!m_pParent->seenCard(p_pCard->suit(), 0));
	
	// If card is not greater than King
	if(pointsOfCardI <= 4)
		points = points + 10 * (!m_pParent->seenCard(p_pCard->suit(), 2));
		
	// If card is not greater than Queen
	if(pointsOfCardI <= 3)
		points = points +  4 * (!m_pParent->seenCard(p_pCard->suit(), 9));
		
	// If card is not greater than Jack
	if(pointsOfCardI <= 2)
		points = points +  3 * (!m_pParent->seenCard(p_pCard->suit(), 8));
	
	// If card is a leak
	if(pointsOfCardI < 2)
		points = points +  2 * (!m_pParent->seenCard(p_pCard->suit(), 7));
	
	return points;
}

bool BKPlayerAI::canTakeWithPointsWithoutBrisKola(BKCard * p_pCardThatCPUCanPlay, BKCard * p_pCardOnTable)
{
	if(
		takesCard(p_pCardThatCPUCanPlay, p_pCardOnTable) &&
		!BKController::match()->isBrisKola(p_pCardThatCPUCanPlay) &&
		(p_pCardThatCPUCanPlay->points() > 0)
	)
		return TRUE;
	else
		return FALSE;
}

unsigned int BKPlayerAI::tryToNotTakeSoft(void)
{
	unsigned int i, res, pointsTaken;
	
	pointsTaken = 10;
	for(i=0; i<3; i++)
	{
		if(_GET_CPU_CARD_(i) == NULL)
			continue;
		
		if(!takesCard(_GET_CPU_CARD_(i), _GET_MATCH_CARD_1) && pointsTaken > _GET_CPU_CARD_(i)->value())
		{ // Looking for the card with minimum value that HUM takes
			res = i;
			pointsTaken = _GET_CPU_CARD_(i)->value();
		}
	}
	
	if(pointsTaken >= 0 && pointsTaken <= 7)
	{ // If the current card is not a load or briskola, CPU choose it
		return res;
	}
	else
	{ // If cards that human takes are all loads and the other ones are briskola
		pointsTaken = 10;
		
		for(unsigned int i = 0; i < 3; i++)
		{
			if(_GET_CPU_CARD_(i) == NULL)
				continue;
			
			if(BKController::match()->isBrisKola(_GET_CPU_CARD_(i)) && pointsTaken > _GET_CPU_CARD_(i)->value())
			{ // Looking for the briskola with minimum value
				res = i;
				pointsTaken = _GET_CPU_CARD_(i)->value();
			}
		}
		
		if(pointsTaken < 10)
		{ // In this case CPU has at least a briskola, so put the briskola with minimum value
			return res;
		}
		else
		{ // In this case all CPU cards are loads, so CPU looks for the load of minimum value
			pointsTaken = 10;
			
			for(unsigned int i = 0; i < 3; i++)
			{
				if(_GET_CPU_CARD_(i) == NULL)
					continue;
				
				if(pointsTaken > _GET_CPU_CARD_(i)->value())
				{ // Looking for the load with minimum value
					res = i;
					pointsTaken = _GET_CPU_CARD_(i)->value();
				}
			}
			
			return res;
		}
	}
}

unsigned int BKPlayerAI::tryToNotTakeHard(void)
{
	unsigned int res, pointsTaken, cardValue;
	
	pointsTaken = 10;
	cardValue = 30; // Indicates the value of the card, where briskola points are increased 5 points
	
	for(unsigned int i = 0; i < 3; i++)
	{
		if(_GET_CPU_CARD_(i) == NULL)
			continue;
		
		if(!takesCard(_GET_CPU_CARD_(i), _GET_MATCH_CARD_1) && cardValue > (_GET_CPU_CARD_(i)->points() + (BKController::match()->isBrisKola(_GET_CPU_CARD_(i)) ? 5 : 0)))
		{ // Looking for the card with minimum value that doesn't take card on table (here briskola cards had a value increased of 5)
			res = i;
			pointsTaken = _GET_CPU_CARD_(i)->value();
			cardValue = (_GET_CPU_CARD_(i)->points() + (BKController::match()->isBrisKola(_GET_CPU_CARD_(i)) ? 5 : 0));
		}
	}
	
	if(pointsTaken >= 0 && pointsTaken <= 7)
	{ // In this case the card found is not a load
		return res;
	}
	else
	{ // In this case cards that human takes are all loads and the other ones are briskola
		pointsTaken = 10;
		
		for(unsigned int i = 0; i < 3; i++)
		{
			if(_GET_CPU_CARD_(i) == NULL)
				continue;
			
			if(BKController::match()->isBrisKola(_GET_CPU_CARD_(i)) && pointsTaken > _GET_CPU_CARD_(i)->value())
			{ // Looking for the briskola with minimum value
				res = i;
				pointsTaken = _GET_CPU_CARD_(i)->value();
			}
		}
		
		if(pointsTaken < 8)
		{ // In this case the briskola (if found!) is a load
			return res;
		}
		else
		{ // In this case all CPU cards are load
			pointsTaken = 10;
			
			for(unsigned int i = 0; i < 3; i++)
			{
				if(_GET_CPU_CARD_(i) == NULL)
					continue;
				
				if(pointsTaken > _GET_CPU_CARD_(i)->value())
				{ // Looking for the card with minimum value
					res = i;
					pointsTaken = _GET_CPU_CARD_(i)->value();
				}
			}
			
			return res;
		}
		
	}
}

unsigned int BKPlayerAI::myMaxPointsHand19ImSecond(BKCard * p_pMyPlayedCard, BKCard * p_pMyOtherCard, BKCard * p_pCardOnTable, BKCard * p_pOppOtherCard)
{
	unsigned int points = 0;
	
	// Start to compute points that I make
	if(takesCard(p_pMyPlayedCard, p_pCardOnTable))
	{
		points = points + p_pMyPlayedCard->points() + p_pCardOnTable->points();
		
		if(!takesCard(p_pOppOtherCard, p_pMyOtherCard))
		{
			points = points + p_pOppOtherCard->points() + p_pMyOtherCard->points();
		}
	}
	else
	{
		if(takesCard(p_pMyOtherCard, p_pOppOtherCard))
		{
			points = points + p_pMyOtherCard->points() + p_pOppOtherCard->points();
		}
	}
	
	return points;
}

unsigned int BKPlayerAI::maxOpponentPointsHand19ImFirst(BKCard * p_pMyCardPlayed, BKCard * p_pMyOtherCard, BKCard * p_pOppCard1, BKCard * p_pOppCard2)
{
	unsigned int points1 = 0, points2 = 0;
	
	if(takesCard(p_pOppCard1, p_pMyCardPlayed))
	{
		points1 = points1 + p_pOppCard1->points() + p_pMyCardPlayed->points();
		
		if(!takesCard(p_pMyOtherCard, p_pOppCard2))
		{
			points1 = points1 + p_pMyOtherCard->points() + p_pOppCard2->points();
		}
	}
	else
	{
		if(takesCard(p_pOppCard2, p_pMyOtherCard))
		{
			points1 = points1 + p_pOppCard2->points() + p_pMyOtherCard->points();
		}
	}
	
	if(takesCard(p_pOppCard2, p_pMyCardPlayed))
	{
		points2 = points2 + p_pOppCard2->points() + p_pMyCardPlayed->points();
		
		if(!takesCard(p_pMyOtherCard, p_pOppCard1))
		{
			points2 = points2 + p_pMyOtherCard->points() + p_pOppCard1->points();
		}
	}
	else
	{
		if(takesCard(p_pOppCard1, p_pMyOtherCard))
		{
			points2 = points2 + p_pOppCard1->points() + p_pMyOtherCard->points();
		}
	}
	
	return _BK_MAX_(points1, points2);
}

unsigned int BKPlayerAI::myMaxPointsHand18ImSecond(BKCard ** p_pMyCards, BKCard ** p_pOppCards)
{ // Note that p_pMyCards[0] is the card that I play and p_pOppCards[0] is the card on table

	unsigned int points = 0, opponentPoints1, opponentPoints2, myPoints1, myPoints2, maxPoints1, maxPoints2;
	unsigned int pointsAvailable;
	
	pointsAvailable = 120 - _GET_CPU_POINTS - _GET_HUM_POINTS;
	
	if(takesCard(p_pMyCards[0], p_pOppCards[0]))
	{
		points = points + p_pMyCards[0]->points() + p_pOppCards[0]->points();
		
		pointsAvailable = pointsAvailable - points;
		
		// In opponentPoints we memorize the max point that can do opponent after this choice
		// opponentPoints1 = maxOpponentPointsHand19ImFirst(p_pMyCards[1], p_pMyCards[2], p_pOppCards[1], p_pOppCards[2]);
		// opponentPoints2 = maxOpponentPointsHand19ImFirst(p_pMyCards[2], p_pMyCards[1], p_pOppCards[1], p_pOppCards[2]);
		
		// In this case I play p_pMyCards[1]
		opponentPoints1 = myMaxPointsHand19ImSecond(p_pOppCards[1], p_pOppCards[2], p_pMyCards[1], p_pMyCards[2]);
		opponentPoints2 = myMaxPointsHand19ImSecond(p_pOppCards[2], p_pOppCards[1], p_pMyCards[1], p_pMyCards[2]);
		myPoints1 = _BK_MAX_(opponentPoints1, opponentPoints2);
		
		// In this case I play p_pMyCards[2]
		opponentPoints1 = myMaxPointsHand19ImSecond(p_pOppCards[1], p_pOppCards[2], p_pMyCards[2], p_pMyCards[1]);
		opponentPoints2 = myMaxPointsHand19ImSecond(p_pOppCards[2], p_pOppCards[1], p_pMyCards[2], p_pMyCards[1]);
		myPoints2 = _BK_MAX_(opponentPoints1, opponentPoints2);
		
		opponentPoints1 = myPoints1;
		opponentPoints2 = myPoints2;
		
		if(opponentPoints1 > opponentPoints2)
			points = points + (pointsAvailable - opponentPoints2);
		else
			points = points + (pointsAvailable - opponentPoints1);
	}
	else
	{
		// Case Opponent plays p_pOppCards[1]
		myPoints1 = myMaxPointsHand19ImSecond(p_pMyCards[1], p_pMyCards[2], p_pOppCards[1], p_pOppCards[2]); // <-| I play p_pMyCards[1]
		myPoints2 = myMaxPointsHand19ImSecond(p_pMyCards[2], p_pMyCards[1], p_pOppCards[1], p_pOppCards[2]); // <-| I play p_pMyCards[2]
		
		maxPoints1 = _BK_MAX_(myPoints1, myPoints2);
		
		// Case Opponent plays p_pOppCards[2]
		myPoints1 = myMaxPointsHand19ImSecond(p_pMyCards[1], p_pMyCards[2], p_pOppCards[2], p_pOppCards[1]); // <-| I play p_pMyCards[1]
		myPoints2 = myMaxPointsHand19ImSecond(p_pMyCards[2], p_pMyCards[1], p_pOppCards[2], p_pOppCards[1]); // <-| I play p_pMyCards[2]
		
		maxPoints2 = _BK_MAX_(myPoints1, myPoints2);
		
		if(maxPoints1 > maxPoints2)
			points = maxPoints2;
		else
			points = maxPoints1;
	}
	
	return points;
}

unsigned int BKPlayerAI::maxOpponentPointsHand18ImFirst(BKCard ** p_pMyCards, BKCard ** p_pOppCards)
{ // This function returns max points that can make opponent if I play p_pMyCards[0]
	unsigned int oppPoints1 = 0, oppPoints2 = 0, oppPoints3 = 0;
	BKCard * temp;
	
	// Max points that opponent can make if plays p_pOppCards[0]
	oppPoints1 = myMaxPointsHand18ImSecond(p_pOppCards, p_pMyCards);
	
	// Max points that opponent can make if play p_pOppCards[1]
	temp = p_pOppCards[0];
	p_pOppCards[0] = p_pOppCards[1];
	p_pOppCards[1] = temp;
	oppPoints2 = myMaxPointsHand18ImSecond(p_pOppCards, p_pMyCards);
	
	// Max points that opponent can make if play p_pOppCards[2]
	temp = p_pOppCards[0];
	p_pOppCards[0] = p_pOppCards[2];
	p_pOppCards[2] = temp;
	oppPoints3 = myMaxPointsHand18ImSecond(p_pOppCards, p_pMyCards);
	
	if(oppPoints1 > oppPoints2)
	{
		if(oppPoints1 > oppPoints3)
			return oppPoints1;
		else
			return oppPoints3;
	}
	else
	{
		if(oppPoints2 > oppPoints3)
			return oppPoints2;
		else
			return oppPoints3;
	}
}

unsigned int BKPlayerAI::myMaxPointsHand17ImSecond(BKCard ** p_pMyCards, BKCard * p_pCardOnTable, BKCard * p_pBrisKola, BKCard ** p_pCardsNotSeen)
{ // Note that p_pMyCards[0] is the card that I play, p_pCardOnTable is the p_pCardOnTable, p_pBrisKola is the last card, p_pCardsNotSeen are the cards not even seen

	BKCard * oppCards[3], * temp, * old[3];
	unsigned int points = 0, points1, points2, points3, maxPoints1, maxPoints2, maxPoints3, minPoints1, minPoints2, minPoints3, maxPoints;
	unsigned int points11, points22, points33;
	unsigned int pointsAvailable;
	unsigned int weight[3];
	
	pointsAvailable = 120 - _GET_HUM_POINTS - _GET_CPU_POINTS;
	
	old[0] = p_pMyCards[0];
	old[1] = p_pMyCards[1];
	old[2] = p_pMyCards[2];

	if(takesCard(p_pMyCards[0], p_pCardOnTable))
	{
		points = points + p_pMyCards[0]->points() + p_pCardOnTable->points();
		
		pointsAvailable = pointsAvailable - points;
		
		weight[0] = 17 - (p_pCardsNotSeen[0]->points() + ((BKController::match()->isBrisKola(p_pCardsNotSeen[0])) ? 5 : 0));
		weight[1] = 17 - (p_pCardsNotSeen[1]->points() + ((BKController::match()->isBrisKola(p_pCardsNotSeen[1])) ? 5 : 0));
		weight[2] = 17 - (p_pCardsNotSeen[2]->points() + ((BKController::match()->isBrisKola(p_pCardsNotSeen[2])) ? 5 : 0));
		
		// In this case I get p_pCardsNotSeen[0], so opponent has the other ones
		p_pMyCards[0] = p_pCardsNotSeen[0];
		oppCards[0] = p_pBrisKola;
		oppCards[1] = p_pCardsNotSeen[1];
		oppCards[2] = p_pCardsNotSeen[2];
		
			// In this case I play p_pMyCards[0]
				// Opponent plays oppCards[0]
				points11 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// Opponent plays oppCards[1]
				temp = oppCards[0];
				oppCards[0] = oppCards[1];
				oppCards[1] = temp;
				points22 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// Opponent plays oppCards[2]
				temp = oppCards[0];
				oppCards[0] = oppCards[2];
				oppCards[2] = temp;
				points33 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// If I play p_pMyCards[0] Opponent try to make the best
				points1 = maxAmong3(points11, points22, points33);
				
			oppCards[0] = p_pBrisKola;
			oppCards[1] = p_pCardsNotSeen[1];
			oppCards[2] = p_pCardsNotSeen[2];
			
			// In this case I play p_pMyCards[1]
			temp = p_pMyCards[0];
			p_pMyCards[0] = p_pMyCards[1];
			p_pMyCards[1] = temp;
				// Opponent plays oppCards[0]
				points11 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// Opponent plays oppCards[1]
				temp = oppCards[0];
				oppCards[0] = oppCards[1];
				oppCards[1] = temp;
				points22 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// Opponent plays oppCards[2]
				temp = oppCards[0];
				oppCards[0] = oppCards[2];
				oppCards[2] = temp;
				points33 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// If I play p_pMyCards[1] Opponent try to make the best
				points2 = maxAmong3(points11, points22, points33);
			
			oppCards[0] = p_pBrisKola;
			oppCards[1] = p_pCardsNotSeen[1];
			oppCards[2] = p_pCardsNotSeen[2];
			
			// In this case I play p_pMyCards[2]
			temp = p_pMyCards[0];
			p_pMyCards[0] = p_pMyCards[2];
			p_pMyCards[2] = temp;
				// Opponent plays oppCards[0]
				points11 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// Opponent plays oppCards[1]
				temp = oppCards[0];
				oppCards[0] = oppCards[1];
				oppCards[1] = temp;
				points22 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// Opponent plays oppCards[2]
				temp = oppCards[0];
				oppCards[0] = oppCards[2];
				oppCards[2] = temp;
				points33 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// If I play p_pMyCards[2] Opponent try to make the best
				points3 = maxAmong3(points11, points22, points33);
			
			// If I get p_pCardsNotSeen[0] I play the cards that minimize opponent's points
			minPoints1 = minAmong3(points1, points2, points3);
		
		p_pMyCards[0] = old[0];
		p_pMyCards[1] = old[1];
		p_pMyCards[2] = old[2];
		
		// In this case I get p_pCardsNotSeen[1], so opponent has the other ones
		p_pMyCards[0] = p_pCardsNotSeen[1];
		oppCards[0] = p_pBrisKola;
		oppCards[1] = p_pCardsNotSeen[0];
		oppCards[2] = p_pCardsNotSeen[2];
		
			// In this case I play p_pMyCards[0]
				// Opponent plays oppCards[0]
				points11 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// Opponent plays oppCards[1]
				temp = oppCards[0];
				oppCards[0] = oppCards[1];
				oppCards[1] = temp;
				points22 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// Opponent plays oppCards[2]
				temp = oppCards[0];
				oppCards[0] = oppCards[2];
				oppCards[2] = temp;
				points33 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// If I play p_pMyCards[0] Opponent try to make the best
				points1 = maxAmong3(points11, points22, points33);
				
			oppCards[0] = p_pBrisKola;
			oppCards[1] = p_pCardsNotSeen[0];
			oppCards[2] = p_pCardsNotSeen[2];
			
			// In this case I play p_pMyCards[1]
			temp = p_pMyCards[0];
			p_pMyCards[0] = p_pMyCards[1];
			p_pMyCards[1] = temp;
				// Opponent plays oppCards[0]
				points11 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// Opponent plays oppCards[1]
				temp = oppCards[0];
				oppCards[0] = oppCards[1];
				oppCards[1] = temp;
				points22 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// Opponent plays oppCards[2]
				temp = oppCards[0];
				oppCards[0] = oppCards[2];
				oppCards[2] = temp;
				points33 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// If I play p_pMyCards[1] Opponent try to make the best
				points2 = maxAmong3(points11, points22, points33);
			
			oppCards[0] = p_pBrisKola;
			oppCards[1] = p_pCardsNotSeen[0];
			oppCards[2] = p_pCardsNotSeen[2];
			
			// In this case I play p_pMyCards[2]
			temp = p_pMyCards[0];
			p_pMyCards[0] = p_pMyCards[2];
			p_pMyCards[2] = temp;
				// Opponent plays oppCards[0]
				points11 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// Opponent plays oppCards[1]
				temp = oppCards[0];
				oppCards[0] = oppCards[1];
				oppCards[1] = temp;
				points22 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// Opponent plays oppCards[2]
				temp = oppCards[0];
				oppCards[0] = oppCards[2];
				oppCards[2] = temp;
				points33 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// If I play p_pMyCards[2] Opponent try to make the best
				points3 = maxAmong3(points11, points22, points33);
			
			// If I get p_pCardsNotSeen[1] I play the cards that minimize opponent's points
			minPoints2 = minAmong3(points1, points2, points3);
		
		p_pMyCards[0] = old[0];
		p_pMyCards[1] = old[1];
		p_pMyCards[2] = old[2];
			
		// In this case I get p_pCardsNotSeen[2], so opponent has the other ones
		p_pMyCards[0] = p_pCardsNotSeen[2];
		oppCards[0] = p_pBrisKola;
		oppCards[1] = p_pCardsNotSeen[1];
		oppCards[2] = p_pCardsNotSeen[0];

			// In this case I play p_pMyCards[0]
				// Opponent plays oppCards[0]
				points11 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// Opponent plays oppCards[1]
				temp = oppCards[0];
				oppCards[0] = oppCards[1];
				oppCards[1] = temp;
				points22 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// Opponent plays oppCards[2]
				temp = oppCards[0];
				oppCards[0] = oppCards[2];
				oppCards[2] = temp;
				points33 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// If I play p_pMyCards[0] Opponent try to make the best
				points1 = maxAmong3(points11, points22, points33);
				
			oppCards[0] = p_pBrisKola;
			oppCards[1] = p_pCardsNotSeen[1];
			oppCards[2] = p_pCardsNotSeen[0];
			
			// In this case I play p_pMyCards[1]
			temp = p_pMyCards[0];
			p_pMyCards[0] = p_pMyCards[1];
			p_pMyCards[1] = temp;
				// Opponent plays oppCards[0]
				points11 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// Opponent plays oppCards[1]
				temp = oppCards[0];
				oppCards[0] = oppCards[1];
				oppCards[1] = temp;
				points22 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// Opponent plays oppCards[2]
				temp = oppCards[0];
				oppCards[0] = oppCards[2];
				oppCards[2] = temp;
				points33 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// If I play p_pMyCards[1] Opponent try to make the best
				points2 = maxAmong3(points11, points22, points33);
			
			oppCards[0] = p_pBrisKola;
			oppCards[1] = p_pCardsNotSeen[1];
			oppCards[2] = p_pCardsNotSeen[0];
			
			// In this case I play p_pMyCards[2]
			temp = p_pMyCards[0];
			p_pMyCards[0] = p_pMyCards[2];
			p_pMyCards[2] = temp;
				// Opponent plays oppCards[0]
				points11 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// Opponent plays oppCards[1]
				temp = oppCards[0];
				oppCards[0] = oppCards[1];
				oppCards[1] = temp;
				points22 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// Opponent plays oppCards[2]
				temp = oppCards[0];
				oppCards[0] = oppCards[2];
				oppCards[2] = temp;
				points33 = myMaxPointsHand18ImSecond(oppCards, p_pMyCards);
				
				// If I play p_pMyCards[2] Opponent try to make the best
				points3 = maxAmong3(points11, points22, points33);
			
			// If I get p_pCardsNotSeen[2] I play the cards that minimize opponent's points
			minPoints3 = minAmong3(points1, points2, points3);
			
		// maxPoints = maxAmong3(minPoints1, minPoints2, minPoints3);
		// maxPoints = (minPoints1 + minPoints2 + minPoints3)/3;
		maxPoints = (minPoints1 * weight[0] + minPoints2 * weight[1] + minPoints3 * weight[2]) / (weight[0] + weight[1] + weight[2]);
		
		points = points + (pointsAvailable - maxPoints);
		
		return points;
	}
	else
	{ // In this case I don't take, so I get the last card. Then opponent's cards are p_pCardsNotSeen

		oppCards[0] = p_pCardsNotSeen[0];
		oppCards[1] = p_pCardsNotSeen[1];
		oppCards[2] = p_pCardsNotSeen[2];
		
		p_pMyCards[0] = p_pBrisKola; // I get the last card
		
		old[0] = p_pMyCards[0];
		old[1] = p_pMyCards[1];
		old[2] = p_pMyCards[2];
		
		// In this case opponent plays oppCards[0]
			// I play p_pMyCards[0]
			points1 = myMaxPointsHand18ImSecond(p_pMyCards, oppCards);
			
			// I play p_pMyCards[1]
			temp = p_pMyCards[0];
			p_pMyCards[0] = p_pMyCards[1];
			p_pMyCards[1] = temp;
			points2 = myMaxPointsHand18ImSecond(p_pMyCards, oppCards);
			
			// I play p_pMyCards[2]
			temp = p_pMyCards[0];
			p_pMyCards[0] = p_pMyCards[2];
			p_pMyCards[2] = temp;
			points3 = myMaxPointsHand18ImSecond(p_pMyCards, oppCards);
		
			if(points1 > points2)
			{
				if(points1 > points3)
					maxPoints1 = points1;
				else
					maxPoints1 = points3;
			}
			else
			{
				if(points2 > points3)
					maxPoints1 = points2;
				else
					maxPoints1 = points3;
			}
		
		// In this case opponent plays oppCards[1]
		temp = oppCards[0];
		oppCards[0] = oppCards[1];
		oppCards[1] = temp;
		
		p_pMyCards[0] = old[0];
		p_pMyCards[1] = old[1];
		p_pMyCards[2] = old[2];
		
			// I play p_pMyCards[0]
			points1 = myMaxPointsHand18ImSecond(p_pMyCards, oppCards);
			
			// I play p_pMyCards[1]
			temp = p_pMyCards[0];
			p_pMyCards[0] = p_pMyCards[1];
			p_pMyCards[1] = temp;
			points2 = myMaxPointsHand18ImSecond(p_pMyCards, oppCards);
			
			// I play p_pMyCards[2]
			temp = p_pMyCards[0];
			p_pMyCards[0] = p_pMyCards[2];
			p_pMyCards[2] = temp;
			points3 = myMaxPointsHand18ImSecond(p_pMyCards, oppCards);
		
			if(points1 > points2)
			{
				if(points1 > points3)
					maxPoints2 = points1;
				else
					maxPoints2 = points3;
			}
			else
			{
				if(points2 > points3)
					maxPoints2 = points2;
				else
					maxPoints2 = points3;
			}
		
		// In this case opponent plays oppCards[2]
		temp = oppCards[0];
		oppCards[0] = oppCards[2];
		oppCards[2] = temp;
		
		p_pMyCards[0] = old[0];
		p_pMyCards[1] = old[1];
		p_pMyCards[2] = old[2];
		
			// I play p_pMyCards[0]
			points1 = myMaxPointsHand18ImSecond(p_pMyCards, oppCards);
			
			// I play p_pMyCards[1]
			temp = p_pMyCards[0];
			p_pMyCards[0] = p_pMyCards[1];
			p_pMyCards[1] = temp;
			points2 = myMaxPointsHand18ImSecond(p_pMyCards, oppCards);
			
			// I play p_pMyCards[2]
			temp = p_pMyCards[0];
			p_pMyCards[0] = p_pMyCards[2];
			p_pMyCards[2] = temp;
			points3 = myMaxPointsHand18ImSecond(p_pMyCards, oppCards);
		
			if(points1 > points2)
			{
				if(points1 > points3)
					maxPoints3 = points1;
				else
					maxPoints3 = points3;
			}
			else
			{
				if(points2 > points3)
					maxPoints3 = points2;
				else
					maxPoints3 = points3;
			}
		
		// We suppose that opponent chooses the card that minimize my maximum points
		if(maxPoints1 < maxPoints2)
		{
			if(maxPoints1 < maxPoints3)
				return maxPoints1;
			else
				return maxPoints3;
		}
		else
		{
			if(maxPoints2 < maxPoints3)
				return maxPoints2;
			else
				return maxPoints3;
		}
	}

}

unsigned int BKPlayerAI::maxOpponentPointsHand17ImFirst(BKCard ** p_pMyCards, BKCard ** p_pCardsNotSeenByMe, BKCard ** p_pCardsNotSeenByOpponent, BKCard * p_pBrisKola)
{ // Note that p_pMyCards[0] is the card that I play and that p_pCardsNotSeenByMe is an array of 4 elements (such as p_pCardsNotSeenByOpponent)
	unsigned int points11, points12, points13;
	unsigned int minPoints1, minPoints2, minPoints3, minPoints4;
	BKCard * presumedOpponentCards[3], * presumedMyCards[3];
	unsigned int weight[4];
	
	weight[0] = 17 - (p_pCardsNotSeenByMe[0]->points() + ((BKController::match()->isBrisKola(p_pCardsNotSeenByMe[0])) ? 5 : 0));
	weight[1] = 17 - (p_pCardsNotSeenByMe[1]->points() + ((BKController::match()->isBrisKola(p_pCardsNotSeenByMe[1])) ? 5 : 0));
	weight[2] = 17 - (p_pCardsNotSeenByMe[2]->points() + ((BKController::match()->isBrisKola(p_pCardsNotSeenByMe[2])) ? 5 : 0));
	weight[3] = 17 - (p_pCardsNotSeenByMe[3]->points() + ((BKController::match()->isBrisKola(p_pCardsNotSeenByMe[3])) ? 5 : 0));
	
	// In this case I suppose that Opponent hasn't p_pCardsNotSeenByMe[0]
		presumedOpponentCards[0] = p_pCardsNotSeenByMe[1];
		presumedOpponentCards[1] = p_pCardsNotSeenByMe[2];
		presumedOpponentCards[2] = p_pCardsNotSeenByMe[3];
		
		// I play p_pMyCards[0]
		for(unsigned int i = 0, j = 0; i < 4; ++i)
		{
			if(p_pCardsNotSeenByOpponent[i] != p_pMyCards[0])
			{
				presumedMyCards[j] = p_pCardsNotSeenByOpponent[i];
				++j;
			}
		}
		points11 = myMaxPointsHand17ImSecond(presumedOpponentCards, p_pMyCards[0], p_pBrisKola, presumedMyCards);
		
		// I play p_pMyCards[1]
		for(unsigned int i = 0, j = 0; i < 4; ++i)
		{
			if(p_pCardsNotSeenByOpponent[i] != p_pMyCards[1])
			{
				presumedMyCards[j] = p_pCardsNotSeenByOpponent[i];
				++j;
			}
		}
		points12 = myMaxPointsHand17ImSecond(presumedOpponentCards, p_pMyCards[1], p_pBrisKola, presumedMyCards);
		
		// I play p_pMyCards[2]
		for(unsigned int i = 0, j = 0; i < 4; ++i)
		{
			if(p_pCardsNotSeenByOpponent[i] != p_pMyCards[2])
			{
				presumedMyCards[j] = p_pCardsNotSeenByOpponent[i];
				++j;
			}
		}
		points13 = myMaxPointsHand17ImSecond(presumedOpponentCards, p_pMyCards[2], p_pBrisKola, presumedMyCards);
		
		// If Opponent hasn't p_pCardsNotSeenByMe[0], I play card that minimizes his points
		minPoints1 = minAmong3(points11, points12, points13);
		
	// In this case I suppose that Opponent hasn't p_pCardsNotSeenByMe[1]
		presumedOpponentCards[0] = p_pCardsNotSeenByMe[0];
		presumedOpponentCards[1] = p_pCardsNotSeenByMe[2];
		presumedOpponentCards[2] = p_pCardsNotSeenByMe[3];
		
		// I play p_pMyCards[0]
		for(unsigned int i = 0, j = 0; i < 4; ++i)
		{
			if(p_pCardsNotSeenByOpponent[i] != p_pMyCards[0])
			{
				presumedMyCards[j] = p_pCardsNotSeenByOpponent[i];
				++j;
			}
		}
		points11 = myMaxPointsHand17ImSecond(presumedOpponentCards, p_pMyCards[0], p_pBrisKola, presumedMyCards);
		
		// I play p_pMyCards[1]
		for(unsigned int i = 0, j = 0; i < 4; ++i)
		{
			if(p_pCardsNotSeenByOpponent[i] != p_pMyCards[1])
			{
				presumedMyCards[j] = p_pCardsNotSeenByOpponent[i];
				++j;
			}
		}
		points12 = myMaxPointsHand17ImSecond(presumedOpponentCards, p_pMyCards[1], p_pBrisKola, presumedMyCards);
		
		// I play p_pMyCards[2]
		for(unsigned int i = 0, j = 0; i < 4; ++i)
		{
			if(p_pCardsNotSeenByOpponent[i] != p_pMyCards[2])
			{
				presumedMyCards[j] = p_pCardsNotSeenByOpponent[i];
				++j;
			}
		}
		points13 = myMaxPointsHand17ImSecond(presumedOpponentCards, p_pMyCards[2], p_pBrisKola, presumedMyCards);
		
		// If Opponent hasn't p_pCardsNotSeenByMe[1], I play card that minimizes his points
		minPoints2 = minAmong3(points11, points12, points13);
		
	// In this case I suppose that Opponent hasn't p_pCardsNotSeenByMe[2]
		presumedOpponentCards[0] = p_pCardsNotSeenByMe[0];
		presumedOpponentCards[1] = p_pCardsNotSeenByMe[1];
		presumedOpponentCards[2] = p_pCardsNotSeenByMe[3];
		
		// I play p_pMyCards[0]
		for(unsigned int i = 0, j = 0; i < 4; ++i)
		{
			if(p_pCardsNotSeenByOpponent[i] != p_pMyCards[0])
			{
				presumedMyCards[j] = p_pCardsNotSeenByOpponent[i];
				++j;
			}
		}
		points11 = myMaxPointsHand17ImSecond(presumedOpponentCards, p_pMyCards[0], p_pBrisKola, presumedMyCards);
		
		// I play p_pMyCards[1]
		for(unsigned int i = 0, j = 0; i < 4; ++i)
		{
			if(p_pCardsNotSeenByOpponent[i] != p_pMyCards[1])
			{
				presumedMyCards[j] = p_pCardsNotSeenByOpponent[i];
				++j;
			}
		}
		points12 = myMaxPointsHand17ImSecond(presumedOpponentCards, p_pMyCards[1], p_pBrisKola, presumedMyCards);
		
		// I play p_pMyCards[2]
		for(unsigned int i = 0, j = 0; i < 4; ++i)
		{
			if(p_pCardsNotSeenByOpponent[i] != p_pMyCards[2])
			{
				presumedMyCards[j] = p_pCardsNotSeenByOpponent[i];
				++j;
			}
		}
		points13 = myMaxPointsHand17ImSecond(presumedOpponentCards, p_pMyCards[2], p_pBrisKola, presumedMyCards);
		
		// If Opponent hasn't p_pCardsNotSeenByMe[2], if play card that minimizes his points
		minPoints3 = minAmong3(points11, points12, points13);
		
	// In this case I suppose that Opponent hasn't p_pCardsNotSeenByMe[3]
		presumedOpponentCards[0] = p_pCardsNotSeenByMe[0];
		presumedOpponentCards[1] = p_pCardsNotSeenByMe[1];
		presumedOpponentCards[2] = p_pCardsNotSeenByMe[2];
		
		// I play p_pMyCards[0]
		for(unsigned int i = 0, j = 0; i < 4; ++i)
		{
			if(p_pCardsNotSeenByOpponent[i] != p_pMyCards[0])
			{
				presumedMyCards[j] = p_pCardsNotSeenByOpponent[i];
				++j;
			}
		}
		points11 = myMaxPointsHand17ImSecond(presumedOpponentCards, p_pMyCards[0], p_pBrisKola, presumedMyCards);
		
		// I play p_pMyCards[1]
		for(unsigned int i = 0, j = 0; i < 4; ++i)
		{
			if(p_pCardsNotSeenByOpponent[i] != p_pMyCards[1])
			{
				presumedMyCards[j] = p_pCardsNotSeenByOpponent[i];
				++j;
			}
		}
		points12 = myMaxPointsHand17ImSecond(presumedOpponentCards, p_pMyCards[1], p_pBrisKola, presumedMyCards);
		
		// I play p_pMyCards[2]
		for(unsigned int i = 0, j = 0; i < 4; ++i)
		{
			if(p_pCardsNotSeenByOpponent[i] != p_pMyCards[2])
			{
				presumedMyCards[j] = p_pCardsNotSeenByOpponent[i];
				++j;
			}
		}
		points13 = myMaxPointsHand17ImSecond(presumedOpponentCards, p_pMyCards[2], p_pBrisKola, presumedMyCards);
		
		// If Opponent hasn't p_pCardsNotSeenByMe[2], if play card that minimizes his points
		minPoints4 = minAmong3(points11, points12, points13);
	
	return ((minPoints1 * weight[0] + minPoints2 * weight[1] + minPoints3 * weight[2] + minPoints4 * weight[3]) / (weight[0] + weight[1] + weight[2] + weight[3]));
	// return (minPoints1 + minPoints2 + minPoints3 + minPoints4)/4;
	// return maxAmong4(minPoints1, minPoints2, minPoints3, minPoints4);
}

unsigned int BKPlayerAI::minAmong3(unsigned int p_uiX, unsigned int p_uiY, unsigned int p_uiZ)
{
	if(p_uiX < p_uiY)
	{
		if(p_uiX < p_uiZ)
			return p_uiX;
		else
			return p_uiZ;
	}
	else
	{
		if(p_uiY < p_uiZ)
			return p_uiY;
		else
			return p_uiZ;
	}
}

unsigned int BKPlayerAI::maxAmong3(unsigned int p_uiX, unsigned int p_uiY, unsigned int p_uiZ)
{
	if(p_uiX > p_uiY)
	{
		if(p_uiX > p_uiZ)
			return p_uiX;
		else
			return p_uiZ;
	}
	else
	{
		if(p_uiY > p_uiZ)
			return p_uiY;
		else
			return p_uiZ;
	}
}

unsigned int BKPlayerAI::maxAmong4(unsigned int p_uiX, unsigned int p_uiY, unsigned int p_uiZ, unsigned int p_uiT)
{
	unsigned int temp;
	
	temp = maxAmong3(p_uiX, p_uiY, p_uiZ);
	
	if(temp > p_uiT)
		return temp;
	else
		return p_uiT;
}
