//=============================================================================
//
//   File : BKPlayerCPU.cpp
//   Created on gio 08 ott 2009 15:01:34 by Iacopo Palazzi
//
//   This file is part of the BrisKola distribution
//   Copyright (C) 2009 Iacopo Palazzi <iacopo.palazzi@gmail.com>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================

#include "BKPlayerCPU.h"

#include <stdlib.h>
#include <time.h>

#include "BK.h"
#include "BKConfig.h"
#include "BKGameBoard.h"
#include "BKDeck.h"
#include "BKPlayerAI.h"

BKPlayerCPU::BKPlayerCPU(BKPlayer::eType p_eType)
	: BKPlayer(p_eType)
{
	m_pAI = new BKPlayerAI(this);
}

BKPlayerCPU::~BKPlayerCPU()
{
	if (m_pAI) delete m_pAI;
}

void BKPlayerCPU::play()
{
	m_pAI->start();
}

void BKPlayerCPU::stop()
{
	m_pAI->quit();
}

void BKPlayerCPU::choosenCard(BKCard * p_pCard)
{
	emit sgnCard(p_pCard);
}

void BKPlayerCPU::assignCard(BKCard * p_pCard)
{
	unsigned int i = 0;
	
	p_pCard->setVisible(true);
	p_pCard->setHidden(!BKConfig::showCPUCards());
	p_pCard->setPlayer(this);
	
	while(m_pCards[i] != NULL && i < 3) { i++; }
	m_pCards[i] = p_pCard;
	
	setSeenCard(p_pCard);
	
	// Reorder CPU cards
	
	bool reorder[3] = {false, false, false};
	BKCard ** newCards = new BKCard * [3];
	newCards[0] = newCards[1] = newCards[2] = NULL;
	
	srand(time(0));
	
	for(unsigned int j = 0, r = 0; j < 3; )
	{
		r = rand() % 3;
		if (reorder[r] == false)
		{
			if (m_pCards[r] != NULL)
			{
				newCards[j] = m_pCards[r];
				BKGameBoard::placeCPUCard(j, newCards[j]);
			}
			reorder[r] = true;
			j++;
		}
	}
	
	delete [] m_pCards;
	m_pCards = newCards;
}

void BKPlayerCPU::takeCard(BKCard * p_pCard)
{
	unsigned int i = 0;
	
	p_pCard->setVisible(false);
	p_pCard->setPlayer(NULL);
	
	while(m_pCards[i] != p_pCard && i < 3) { i++; }
	m_pCards[i] = NULL;
}

