//=============================================================================
//
//   File : BKHelp.cpp
//   Created on mar 06 ott 2009 14:51:56 by Iacopo Palazzi
//
//   This file is part of the BrisKola distribution
//   Copyright (C) 2009 Iacopo Palazzi <iacopo.palazzi@gmail.com>
//   Copyright (C) 2009 Gianni Sammito <giannixsammito@gmail.com>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================

#include "BKGlobal.h"

#include "BKHelp.h"

#include <QTextEdit>
#include <QDialogButtonBox>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QTabWidget>
#include <QString>

BKHelp::BKHelp(QWidget * p_pParent)
: QDialog(p_pParent)
{
	m_pTabWidget = NULL;
	m_pRules = NULL;
	m_pHowToPlay = NULL;
	m_pButtonBox = NULL;
	m_pVLayout = NULL;
	m_pFooter = NULL;
	m_pTextBoxRules = NULL;
	m_pTextBoxHowToPlay = NULL;
	
	setWindowModality(Qt::WindowModal);
	setWindowTitle(QString("Help"));
	
	m_pVLayout = new QVBoxLayout();
	m_pFooter = new QHBoxLayout;
	m_pFooter->setAlignment(Qt::AlignRight);
	m_pTabWidget = new QTabWidget();
	m_pButtonBox = new QDialogButtonBox(Qt::Horizontal, this);
	m_pButtonBox->addButton(QDialogButtonBox::Close);
	
	createRules();
	createHowToPlay();
	
	m_pTabWidget->addTab(m_pTextBoxRules, QString("Regole"));
	m_pTabWidget->addTab(m_pTextBoxHowToPlay, QString("Come giocare"));
	
	m_pFooter->addWidget(m_pButtonBox);
	
	m_pVLayout->addWidget(m_pTabWidget);
	m_pVLayout->addLayout(m_pFooter);
	
	setLayout(m_pVLayout);
	
	setFixedSize(330, 390);
	
	connect(m_pButtonBox, SIGNAL(clicked(QAbstractButton *)), this, SLOT(sltClose()));
}

BKHelp::~BKHelp()
{
	if(m_pHowToPlay) delete m_pHowToPlay;
	if(m_pRules) delete m_pRules;
	if(m_pTextBoxHowToPlay) delete m_pTextBoxHowToPlay;
	if(m_pTextBoxRules) delete m_pTextBoxRules;
	if(m_pButtonBox) delete m_pButtonBox;
	if(m_pFooter) delete m_pFooter;
	if(m_pTabWidget) delete m_pTabWidget;
	if(m_pVLayout) delete m_pVLayout;
}


// ================================
// Start putting your stuff here...
// ================================

void BKHelp::setRulesText()
{
	m_pRules->setHtml(QString("<strong>Regole della BrisKola</strong><div align=\"justify\">Nel gioco della briscola a due giocatori si inizia scegliendo casualmente il mazziere che distribuisce tre carte per giocatore, quindi pone la settima carta rovesciata sul tavolo con sopra il mazzo delle carte rimanenti, lasciando per&ograve; parzialmente visibile la carta rovesciata. Il seme di tale carta &egrave; la briscola.<br />Inizia il secondo giocatore buttando una carta quindi il mazziere risponde con un'altra carta. La seconda carta giocata batte la prima se e soltanto se si verifica una di queste due condizioni:<ul><li>La seconda carta &egrave; una briscola mentre la prima no</li><li>Le due carte sono dello stesso seme ma la seconda ha una valore maggiore</li></ul>Di seguito viene riportato, in ordine decrescente, il valore delle carte<ul><li>Asso</li><li>3</li><li>Re</li><li>Regina</li><li>Fante</li><li>7</li><li>6</li><li>5</li><li>4</li><li>2</li></ul>Il giocatore che vince la mano prende le due carte giocate e pesca una nuova carta, dopodich&eacute; anche l'altro giocatore pesca una carta dal mazzo. Ad ogni mano gioca per primo il giocatore che ha vinto la mano precedente, quindi il gioco procede nello stesso modo fino alla fine.<br />Una volta esaurite le carte la partita finisce e si procede con il conteggio dei punti totalizzati da ogni giocatore in base alle carte che ha conquistato. I punti associati alle carte sono i seguenti<br /><center><table border=\"1\"><tr><td>Carta</td><td>Punti</td></tr><tr><td>Asso</td><td align=\"right\">11</td></tr><tr><td>3</td><td align=\"right\">10</td></tr><tr><td>Re</td><td align=\"right\">4</td></tr><tr><td>Regina</td><td align=\"right\">3</td></tr><tr><td>Fante</td><td align=\"right\">2</td></tr><tr><td>Tutte le altre</td><td align=\"right\">0</td></tr></table></center><br />Il massimo punteggio ottenibile &egrave; 120, di conseguenza per vincere &egrave; sufficiente totalizzare un punteggio maggiore di 60. Nel caso in cui entrambi i giocatori totalizzino 60 punti la partita finisce in parit&agrave;. Per approfondimenti visita questa pagina:<br />http://www.briskola.net/regole.php</div>"));
}

void BKHelp::setHowToPlayText()
{
	m_pHowToPlay->setHtml(QString("<strong>Come giocare a BrisKola</strong><div align=\"justify\">Per iniziare una nuova partita cliccare sul bottone Nuova Partita, scegliere Nuova Partita dal menu BrisKola oppure premere F2. Per giocare una carta &egrave; sufficiente cliccarci sopra oppure usare sfruttare la seguente combinazione di tasti<br /><center><table border=\"1\"><tr><td>Carta</td><td>Tasto</td></tr><tr><td>Sinistra</td><td align=\"right\">1</td></tr><tr><td>Centrale</td><td align=\"right\">2</td></tr><tr><td>Destra</td><td align=\"right\">3</td></tr></table></center><br />Tale combinazione &egrave; in ogni caso modificabile dalle opzioni del gioco (Opzioni... dal menu BrisKola).</div>"));
}

void BKHelp::createRules()
{
	m_pTextBoxRules = new QWidget();
	m_pRules = new QTextEdit(m_pTextBoxRules);
	
	setRulesText();
	m_pRules->setReadOnly(true);
	m_pRules->setFixedSize(300, 300);
}

void BKHelp::createHowToPlay()
{
	m_pTextBoxHowToPlay = new QWidget();
	m_pHowToPlay = new QTextEdit(m_pTextBoxHowToPlay);
	
	setHowToPlayText();
	m_pHowToPlay->setReadOnly(true);
	m_pHowToPlay->setFixedSize(300, 300);
}

void BKHelp::sltClose()
{
	hide();
}
