//=============================================================================
//
//   File : BKMainWindow.cpp
//   Created on mar 06 ott 2009 14:51:56 by Iacopo Palazzi
//
//   This file is part of the BrisKola distribution
//   Copyright (C) 2009 Iacopo Palazzi <iacopo.palazzi@gmail.com>
//   Copyright (C) 2009 Gianni Sammito <giannixsammito@gmail.com>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================

#include "BKMainWindow.h"

#include "BK.h"
#include "BKApplication.h"
#include "BKConfig.h"
#include "BKController.h"
#include "BKGameBoard.h"
#include "BKAbout.h"
#include "BKOptions.h"
#include "BKHelp.h"
#include "BKAlertNotNewVersion.h"
#include "BKAlertNewVersion.h"
#include "BKAlertConnectError.h"

#include <QAction>
#include <QDesktopWidget>
#include <QMenu>
#include <QMenuBar>
#include <QGridLayout>
#include <QLabel>
#include <QPushButton>
#include <QToolBar>
#include <QVBoxLayout>
#include <QWidget>
#include <QDesktopServices>
#include <QUrl>


// The one and only BKMainWindowobject
BKMainWindow * BKMainWindow::m_pInstance = NULL;

BKMainWindow::BKMainWindow()
: QMainWindow()
{
	m_pMenuFile = NULL;
	m_pMenuHelp = NULL;
	m_pMenuBar = NULL;
	m_pActionNewGame = NULL;
	m_pActionOptions = NULL;
	m_pActionExit = NULL;
	m_pActionShowCards = NULL;
	m_pActionHelp = NULL;
	m_pActionAbout = NULL;
	m_pActionWebSite = NULL;
	m_pActionVersion = NULL;
	m_pLayout = NULL;
	m_pToolBar = NULL;
	
	m_pAbout = NULL;
	m_pOptions = NULL;
	m_pHelp = NULL;
	m_pNotNewVersion = NULL;
	m_pNewVersion = NULL;
	m_pConnectionError = NULL;
	
	m_bWindowsShow = false;
	
	createActions();
	createMenuBar();
	createToolBar();
	createGraphics();
	
	setWindowIcon(QIcon(QPixmap(BKConfig::getImage(BKConfig::IMG_ICON_APP))));
}

BKMainWindow::~BKMainWindow()
{
	if (m_pMenuFile) delete m_pMenuFile;
	if (m_pMenuHelp) delete m_pMenuHelp;
	if (m_pMenuBar) delete m_pMenuBar;
	if (m_pActionNewGame) delete m_pActionNewGame;
	if (m_pActionExit) delete m_pActionExit;
	if (m_pActionOptions) delete m_pActionOptions;
	if (m_pActionShowCards) delete m_pActionShowCards;
	if (m_pActionHelp) delete m_pActionHelp;
	if (m_pActionAbout) delete m_pActionAbout;
	if (m_pActionWebSite) delete m_pActionWebSite;
	if (m_pActionVersion) delete m_pActionVersion;
	if (m_pLayout) delete m_pLayout;
	if (m_pToolBar) delete m_pToolBar;
	
	if (m_pConnectionError) delete m_pConnectionError;
	if (m_pNotNewVersion) delete m_pNotNewVersion;
	if (m_pNewVersion) delete m_pNewVersion;
	if (m_pHelp) delete m_pHelp;
	if (m_pAbout) delete m_pAbout;
	if (m_pOptions) delete m_pOptions;
	
	BKGameBoard::done();
}

bool BKMainWindow::init()
{
	if(m_pInstance)return false;
	m_pInstance = new BKMainWindow();
	return true;
}

void BKMainWindow::done()
{
	if(!m_pInstance)return;
	delete m_pInstance;
	m_pInstance = NULL;
}

void BKMainWindow::checkVersion(unsigned int p_uiStatus)
{ // p_uiStatus: 0, this is the last version; 1, this isn't the last version; 2, connection error

	switch(p_uiStatus)
	{
		case 0:
			if(!m_pInstance->m_pNotNewVersion)
				m_pInstance->m_pNotNewVersion = new BKAlertNotNewVersion(m_pInstance);
			m_pInstance->m_pNotNewVersion->show();
		break;
		case 1:
			if(!m_pInstance->m_pNewVersion)
				m_pInstance->m_pNewVersion = new BKAlertNewVersion(m_pInstance);
			m_pInstance->m_pNewVersion->show();
		break;
		case 2:
			if(!m_pInstance->m_pConnectionError)
				m_pInstance->m_pConnectionError = new BKAlertConnectError(m_pInstance);
			m_pInstance->m_pConnectionError->show();
		break;		
	}
}

// ================================
// Start putting your stuff here...
// ================================

void BKMainWindow::showEvent ( QShowEvent * event )
{
	centerWindowOneTime();
}

void BKMainWindow::centerWindow(bool p_bOneTime = false)
{
	if (p_bOneTime && m_bWindowsShow) return;
	m_bWindowsShow = true;
	
	QRect geom(frameGeometry());
	
	unsigned int uiDeskWidth = BKApplication::desktop()->screenGeometry().width();
	unsigned int uiDeskHeight = BKApplication::desktop()->screenGeometry().height();
	unsigned int uiWinWidth = geom.width();
	unsigned int uiWinHeight = geom.height();
	
	geom.setX((uiDeskWidth - uiWinWidth) / 2);
	geom.setY((uiDeskHeight - uiWinHeight) / 2);
	
	setGeometry(geom);
}

void BKMainWindow::createActions()
{
	m_pActionExit = new QAction(QIcon(BKConfig::getImage(BKConfig::IMG_ICON_CLOSE)), tr("&Esci"), this);
		m_pActionExit->setShortcut(tr("Ctrl+Q"));
		m_pActionExit->setStatusTip(tr("Exit from BrisKola"));
		
	m_pActionNewGame = new QAction(QIcon(BKConfig::getImage(BKConfig::IMG_ICON_START)), tr("&Nuova partita"), this);
		m_pActionNewGame->setShortcut(tr("F2"));
		m_pActionNewGame->setStatusTip(tr("Start New Game"));
		
	m_pActionOptions = new QAction(QIcon(BKConfig::getImage(BKConfig::IMG_ICON_SETTINGS)), tr("&Opzioni..."), this);
		m_pActionOptions->setShortcut(tr(""));
		m_pActionOptions->setStatusTip(tr("Opzioni"));
		
	#ifdef DEBUG
	m_pActionShowCards = new QAction(tr("&Show Cards"), this);
		m_pActionShowCards->setShortcut(tr("Ctrl+S"));
		m_pActionShowCards->setStatusTip(tr("Show/Hide Opponent's Cards"));
	#endif
		
	m_pActionHelp = new QAction(tr("&Aiuto"), this);
		m_pActionHelp->setShortcut(tr("F1"));
		m_pActionHelp->setStatusTip(tr("Ottieni aiuto"));
		
	m_pActionVersion = new QAction(tr("&Controllo versione..."), this);
		m_pActionHelp->setStatusTip(trUtf8("Controlla se sono disponibili versioni più recenti"));
		
	m_pActionWebSite = new QAction(tr("&BrisKola sul web"), this);
		m_pActionWebSite->setShortcut(tr(""));
		m_pActionWebSite->setStatusTip(tr("Visita briskola.net"));
	
	m_pActionAbout = new QAction(tr("&About"), this);
		m_pActionAbout->setShortcut(tr(""));
		m_pActionAbout->setStatusTip(tr("Chi siamo"));
		
	connect(m_pActionNewGame, SIGNAL(triggered()), this, SLOT(sltNewGame()));
	connect(m_pActionExit, SIGNAL(triggered()), this, SLOT(sltExit()));

	#ifdef DEBUG
		connect(m_pActionShowCards, SIGNAL(triggered()), this, SLOT(sltShowCards()));
	#endif

	connect(m_pActionWebSite, SIGNAL(triggered()), this, SLOT(sltWebSite()));
	connect(m_pActionAbout, SIGNAL(triggered()), this, SLOT(sltAbout()));
	connect(m_pActionVersion, SIGNAL(triggered()), this, SLOT(sltVersion()));
	connect(m_pActionOptions, SIGNAL(triggered()), this, SLOT(sltOptions()));
	connect(m_pActionHelp, SIGNAL(triggered()), this, SLOT(sltHelp()));
}

void BKMainWindow::createToolBar()
{
	m_pToolBar = new QToolBar(tr("&BrisKola"), this);
		m_pToolBar->addAction(m_pActionNewGame);
		m_pToolBar->addAction(m_pActionExit);

		#ifdef DEBUG
			m_pToolBar->addAction(m_pActionShowCards);
		#endif
		
		m_pToolBar->setFloatable(false);
		m_pToolBar->setMovable(false);
}

void BKMainWindow::createMenuBar()
{
	m_pMenuFile = new QMenu(tr("&BrisKola"));
		m_pMenuFile->addAction(m_pActionNewGame);
		m_pMenuFile->addAction(m_pActionOptions);
		m_pMenuFile->addAction(m_pActionExit);
		
	m_pMenuHelp = new QMenu(tr("&?"));
		m_pMenuHelp->addAction(m_pActionHelp);
		m_pMenuHelp->addAction(m_pActionVersion);
		m_pMenuHelp->addAction(m_pActionWebSite);
		m_pMenuHelp->addAction(m_pActionAbout);
	
	m_pMenuBar = new QMenuBar();
		m_pMenuBar->addMenu(m_pMenuFile);
		m_pMenuBar->addMenu(m_pMenuHelp);
}

void BKMainWindow::createGraphics()
{
	setMenuBar(m_pMenuBar);
	addToolBar(m_pToolBar);
	
	BKGameBoard::init(this);
	
	setCentralWidget(BKGameBoard::instance());
	setWindowTitle(BKConfig::getString(BKConfig::TXT_APPNAME) + " - " + BKConfig::getString(BKConfig::TXT_VERSION_NAME) + " (" + BKConfig::getString(BKConfig::TXT_VERSION) + ")");
	centralWidget()->setFixedSize(BKConfig::getDisplayRect().width() + 4, BKConfig::getDisplayRect().height() + 4);
	
	layout()->setSizeConstraint(QLayout::SetFixedSize);
}

void BKMainWindow::newGame()
{
	dbg("Starting New Game!!");
	BKGameBoard::instance()->doSetVisiblePoints(BKConfig::instance()->getVisiblePoints(true));
	BKGameBoard::instance()->hideTxtWinner();
	BKController::instance()->startGame();
}

void BKMainWindow::exit()
{
	dbg("Exiting from BrisKola...");
	BKApplication::instance()->quit();
}

void BKMainWindow::showCPUCards()
{
	BKConfig::showCPUCards(!BKConfig::showCPUCards());
	BKController::showCPUCards(BKConfig::showCPUCards());
}

void BKMainWindow::sltNewGame()
{
	newGame();
}

void BKMainWindow::sltExit()
{
	exit();
}

void BKMainWindow::sltShowCards()
{
	showCPUCards();
	
	if (BKConfig::showCPUCards())
		m_pActionShowCards->setText("&Hide Cards");
	else
		m_pActionShowCards->setText("&Show Cards");
}

void BKMainWindow::sltWebSite()
{
	QDesktopServices::openUrl(QUrl::fromEncoded("http://www.briskola.net"));
}

void BKMainWindow::sltAbout()
{
	printf("qui 1");
	if(!m_pAbout)
		m_pAbout = new BKAbout(this);
	m_pAbout->show();
}

void BKMainWindow::sltOptions()
{
	if(!m_pOptions)
		m_pOptions = new BKOptions(this);
		
	//m_pOptions->setKeysToCurrentValue();
	m_pOptions->setCurrentOptionsChecked();
	m_pOptions->show();	
}

void BKMainWindow::sltHelp()
{
	if(!m_pHelp)
		m_pHelp = new BKHelp(this);
	m_pHelp->show();
}

void BKMainWindow::sltVersion()
{
	// Put here instructions to check version...
	BKConfig::checkVersion();
}


