#ifndef _BKMAINWINDOW_H_
#define _BKMAINWINDOW_H_
//=============================================================================
//
//   File : BKMainWindow.h
//   Created on mar 06 ott 2009 14:51:56 by Iacopo Palazzi
//
//   This file is part of the BrisKola distribution
//   Copyright (C) 2009 Iacopo Palazzi <iacopo.palazzi@gmail.com>
//   Copyright (C) 2009 Gianni Sammito <giannixsammito@gmail.com>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================

#include <QMainWindow>

class QAction;
class QDesktopServices;
class QGridLayout;
class QMenu;
class QMenuBar;
class QToolBar;
class QShowEvent;
class QUrl;

class BKAbout;
class BKOptions;
class BKHelp;
class BKAlertNotNewVersion;
class BKAlertNewVersion;
class BKAlertConnectError;

class BKMainWindow : public QMainWindow
{
	Q_OBJECT
protected:
	BKMainWindow();
	~BKMainWindow();

// ================================
// Put members declaration below...
// ================================
private:
	QAction * m_pActionNewGame;
	QAction * m_pActionOptions;
	QAction * m_pActionExit;
	QAction * m_pActionShowCards;
	QAction * m_pActionHelp;
	QAction * m_pActionAbout;
	QAction * m_pActionWebSite;
	QAction * m_pActionVersion;

	QToolBar * m_pToolBar;

	QMenuBar * m_pMenuBar;
	QMenu * m_pMenuFile;
	QMenu * m_pMenuHelp;

	QGridLayout * m_pLayout;
	
	BKAbout * m_pAbout;
	BKOptions * m_pOptions;
	BKHelp * m_pHelp;
	BKAlertNotNewVersion * m_pNotNewVersion;
	BKAlertNewVersion * m_pNewVersion;
	BKAlertConnectError * m_pConnectionError;

	bool m_bWindowsShow;

protected:

public:


// ================================
// Put methods declaration below...
// ================================
private:
	void createActions();
	void createMenuBar();
	void createToolBar();
	void createGraphics();

	void newGame();
	void exit();
	void showCPUCards();

	void centerWindow(bool p_bOneTime);
	void centerWindowOneTime() { centerWindow(true); };
	void showEvent ( QShowEvent * event );

protected:

public:

private slots:
	void sltNewGame();
	void sltExit();
	void sltShowCards();
	void sltWebSite();
	void sltAbout();
	void sltOptions();
	void sltHelp();
	void sltVersion();

// Class follows singleton pattern
protected:
	static BKMainWindow * m_pInstance;
public:
	static BKMainWindow * instance() { return m_pInstance; };
	static bool init();
	static void done();
	static void checkVersion(unsigned int p_uiStatus);

};

#endif //!_BKMAINWINDOW_H_
