//=============================================================================
//
//   File : BKOptions.cpp
//   Created on mar 06 ott 2009 14:51:56 by Iacopo Palazzi
//
//   This file is part of the BrisKola distribution
//   Copyright (C) 2009 Iacopo Palazzi <iacopo.palazzi@gmail.com>
//   Copyright (C) 2009 Gianni Sammito <giannixsammito@gmail.com>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================

#include "BKGameBoard.h"
#include "BKOptions.h"
#include "BKGlobal.h"
#include "BKConfig.h"
#include "BK.h"

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QString>
#include <QRadioButton>
#include <QLabel>
#include <QPixmap>
#include <QTabWidget>
#include <QGridLayout>
#include <QComboBox>
#include <QSlider>
#include <QSpinBox>
#include <QCheckBox>
#include <QDialogButtonBox>

#include <stdlib.h>

BKOptions::BKOptions(QWidget * p_pParent)
: QDialog(p_pParent)
{
	m_pMainTab = NULL;
	m_pMainLayout = NULL;
	m_pDeckTab = NULL;
	m_pBackTab = NULL;
	m_pKeyTab = NULL;
	m_pSpeedTab = NULL;
	m_pGeneralTab = NULL;
	m_pKeyLayout = NULL;
	m_pVLayoutLeft = NULL;
	m_pVLayoutRight = NULL;
	m_pHLayout = NULL;
	m_pHBackLayout = NULL;
	m_pVBackLayoutLeft = NULL;
	m_pVBackLayoutRight = NULL;
	m_pBackLayout = NULL;
	m_pDeckLayout = NULL;
	m_pSpeedLayout = NULL;
	m_pGeneralLayout = NULL;
	m_pSpeedFooter = NULL;
	m_pFooter = NULL;
	m_pButtonBox = NULL;
	m_pPreview = NULL;
	m_pBackPreview = NULL;
	m_pDeckImage = NULL;
	m_pBackImage = NULL;
	m_pLabelImage = NULL;
	m_pLabelBackImage = NULL;
	m_pLabelKey1 = NULL;
	m_pLabelKey2 = NULL;
	m_pLabelKey3 = NULL;
	m_pRadioSpeedHigh = NULL;
	m_pRadioSpeedNormal = NULL;
	m_pRadioSpeedSlow = NULL;
	m_pRadioSpeedManual = NULL;
	m_pSliderSpeed = NULL;
	m_pSpinBoxManualSpeed = NULL;
	m_pCheckDelayCPU = NULL;
	m_pComboBoxKey[0] = NULL;
	m_pComboBoxKey[1] = NULL;
	m_pComboBoxKey[2] = NULL;
	m_pRadioGeneral[0] = NULL;
	m_pRadioGeneral[1] = NULL;
	m_pRadioGeneral[2] = NULL;
	m_pCheckBoxGeneralPoints = NULL;
	m_pCheckBoxGeneralSquare = NULL;
	
	m_pDeckRadioButton = NULL;
	m_pBackRadioButton = NULL;
	
	m_pWindowAlert = NULL;
	
	setWindowModality(Qt::WindowModal);
	setWindowTitle(QString("Opzioni"));
	
	m_pMainLayout = new QVBoxLayout();
	
	createDeckTab();
	createBackTab();
	createKeyTab();
	createSpeedTab();
	createGeneralTab();
	
	m_pFooter = new QHBoxLayout();
	m_pFooter->setAlignment(Qt::AlignRight);
	
	m_pButtonBox = new QDialogButtonBox(Qt::Horizontal, this);
		m_pButtonBox->addButton(QDialogButtonBox::Cancel);
		m_pButtonBox->addButton(QDialogButtonBox::Ok);
	
	m_pFooter->addWidget(m_pButtonBox);
	
	m_pDeckTab->setLayout(m_pDeckLayout);
	m_pBackTab->setLayout(m_pBackLayout);
	
	m_pMainTab = new QTabWidget(this);
		m_pMainTab->addTab(m_pDeckTab, "Mazzo");
		m_pMainTab->addTab(m_pBackTab, "Sfondo");
		m_pMainTab->addTab(m_pKeyTab, "Tasti");
		m_pMainTab->addTab(m_pSpeedTab, trUtf8("Velocità"));
		m_pMainTab->addTab(m_pGeneralTab, "Generale");
	
	m_pMainLayout->addWidget(m_pMainTab);
	m_pMainLayout->addLayout(m_pFooter);
	
	//m_pMainLayout->setSizeConstraint(QLayout::SetFixedSize);
	
	setLayout(m_pMainLayout);
	
	//setWindowIcon(QIcon(QPixmap(_STR_DIR_IMG_MISC + QString("/bkIcon.png"))));
	
	connect(m_pButtonBox, SIGNAL(accepted()), this, SLOT(sltButtonOk()));
	connect(m_pButtonBox, SIGNAL(rejected()), this, SLOT(sltButtonReset()));
	
	setFixedWidth(350);
}

BKOptions::~BKOptions()
{
	if(m_pDeckRadioButton)
	{
		for(int i = 6; i >= 0; --i)
			if(m_pDeckRadioButton[i]) delete m_pDeckRadioButton[i];
		
		free(m_pDeckRadioButton);
	}
	
	if(m_pBackRadioButton)
	{
		for(int i = 6; i >= 0; --i)
			if(m_pBackRadioButton[i]) delete m_pBackRadioButton[i];
		
		free(m_pBackRadioButton);
	}
	
	if(m_pWindowAlert) delete m_pWindowAlert;
	if(m_pCheckBoxGeneralPoints) delete m_pCheckBoxGeneralPoints;
	if(m_pCheckBoxGeneralSquare) delete m_pCheckBoxGeneralSquare;
	if(m_pRadioGeneral[0]) delete m_pRadioGeneral[0];
	if(m_pRadioGeneral[1]) delete m_pRadioGeneral[1];
	if(m_pRadioGeneral[2]) delete m_pRadioGeneral[2];
	if(m_pComboBoxKey[0]) delete m_pComboBoxKey[0];
	if(m_pComboBoxKey[1]) delete m_pComboBoxKey[1];
	if(m_pComboBoxKey[2]) delete m_pComboBoxKey[2];
	if(m_pCheckDelayCPU) delete m_pCheckDelayCPU;
	if(m_pSpinBoxManualSpeed) delete m_pSpinBoxManualSpeed;
	if(m_pSliderSpeed) delete m_pSliderSpeed;
	if(m_pRadioSpeedHigh) delete m_pRadioSpeedHigh;
	if(m_pRadioSpeedNormal) delete m_pRadioSpeedNormal;
	if(m_pRadioSpeedSlow) delete m_pRadioSpeedSlow;
	if(m_pRadioSpeedManual) delete m_pRadioSpeedManual;
	if(m_pLabelKey1) delete m_pLabelKey1;
	if(m_pLabelKey2) delete m_pLabelKey2;
	if(m_pLabelKey3) delete m_pLabelKey3;
	if(m_pDeckImage) delete m_pDeckImage;
	if(m_pBackImage) delete m_pBackImage;
	if(m_pLabelImage) delete m_pLabelImage;
	if(m_pLabelBackImage) delete m_pLabelBackImage;
	if(m_pPreview) delete m_pPreview;
	if(m_pBackPreview) delete m_pBackPreview;
	if(m_pVBackLayoutRight) delete m_pVBackLayoutRight;
	if(m_pVBackLayoutLeft) delete m_pVBackLayoutLeft;
	if(m_pVLayoutRight) delete m_pVLayoutRight;
	if(m_pVLayoutLeft) delete m_pVLayoutLeft;
	if(m_pHLayout) delete m_pHLayout;
	if(m_pHBackLayout) delete m_pHBackLayout;
	if(m_pSpeedFooter) delete m_pSpeedFooter;
	if(m_pGeneralLayout) delete m_pGeneralLayout;
	if(m_pSpeedLayout) delete m_pSpeedLayout;
	if(m_pKeyLayout) delete m_pKeyLayout;
	if(m_pButtonBox) delete m_pButtonBox;
	if(m_pFooter) delete m_pFooter;
	if(m_pDeckLayout) delete m_pDeckLayout;
	if(m_pBackLayout) delete m_pBackLayout;
	if(m_pGeneralTab) delete m_pGeneralTab;
	if(m_pSpeedTab) delete m_pSpeedTab;
	if(m_pKeyTab) delete m_pKeyTab;
	if(m_pDeckTab) delete m_pDeckTab;
	if(m_pBackTab) delete m_pBackTab;
	if(m_pMainLayout) delete m_pMainLayout;
	if(m_pMainTab) delete m_pMainTab;
}

// ================================
// Start putting your stuff here...
// ================================

void BKOptions::createDeckTab()
{
	m_pDeckTab = new QWidget();
	
	m_pVLayoutLeft = new QVBoxLayout();
	m_pVLayoutRight = new QVBoxLayout();
	m_pHLayout = new QHBoxLayout();
	
	m_pDeckLayout = new QVBoxLayout();
	
	m_pPreview = new QLabel(tr("Anteprima"));
	m_pLabelImage = new QLabel();
	
	int iDeck = BKConfig::getDeck(1);
	
	m_pDeckImage = new QPixmap(_STR_DIR_IMG_DECKS + QString("/bkImgDeck_0" + QString("%1").setNum(iDeck) + ".png"));
	
	m_pLabelImage->setPixmap(* m_pDeckImage);
	
	m_pDeckRadioButton = (QRadioButton **)malloc(7 * sizeof(QRadioButton *));
	
	m_pDeckRadioButton[0] = new QRadioButton(QString("Mazzo KDE"));
	m_pDeckRadioButton[1] = new QRadioButton(QString("Mazzo BK arancione"));
	m_pDeckRadioButton[2] = new QRadioButton(QString("Mazzo BK blu"));
	m_pDeckRadioButton[3] = new QRadioButton(QString("Mazzo BK viola"));
	m_pDeckRadioButton[4] = new QRadioButton(QString("Mazzo Linux"));
	m_pDeckRadioButton[5] = new QRadioButton(QString("Mazzo Windows"));
	m_pDeckRadioButton[6] = new QRadioButton(QString("Mazzo Mac"));
	
	for(unsigned int i = 0; i < 7; ++i)
		m_pVLayoutLeft->addWidget(m_pDeckRadioButton[i]);
	
	for(unsigned int i = 0; i < 7; ++i)
		connect(m_pDeckRadioButton[i], SIGNAL(toggled(bool)), this, SLOT(sltDeck(bool)));
		
	m_pVLayoutRight->addWidget(m_pPreview);
	m_pVLayoutRight->addWidget(m_pLabelImage);
	
	m_pVLayoutRight->setAlignment(Qt::AlignTop);
	
	m_pHLayout->addLayout(m_pVLayoutLeft);
	m_pHLayout->addLayout(m_pVLayoutRight);
	
	m_pDeckLayout->addLayout(m_pHLayout);
}

void BKOptions::createBackTab()
{
	m_pBackTab = new QWidget();
	
	m_pVBackLayoutLeft = new QVBoxLayout();
	m_pVBackLayoutRight = new QVBoxLayout();
	m_pHBackLayout = new QHBoxLayout();
	
	m_pBackLayout = new QVBoxLayout();
	
	m_pBackPreview = new QLabel(tr("Anteprima"));
	m_pLabelBackImage = new QLabel();
	
	int iBack = BKConfig::getBackground(0);
	
	if (iBack < 0 || iBack > 6) iBack = 0;
	
	m_pBackImage = new QPixmap(_STR_DIR_IMG_BACKGROUND + QString("/bkImgBackground_0" + QString("%1").setNum(iBack) + "_small.png"));
	
	m_pLabelBackImage->setPixmap(* m_pBackImage);
	
	//dbg((_STR_DIR_IMG_BACKGROUND + QString("/bkImgBackground_0" + QString(iBack) + "_small.png")).toStdString().c_str());
	
	m_pBackRadioButton = (QRadioButton **)malloc(7 * sizeof(QRadioButton *));
	
	m_pBackRadioButton[0] = new QRadioButton(QString("Classic"));
	m_pBackRadioButton[1] = new QRadioButton(QString("Hawai"));
	m_pBackRadioButton[2] = new QRadioButton(QString("Space"));
	m_pBackRadioButton[3] = new QRadioButton(QString("Nature"));
	m_pBackRadioButton[4] = new QRadioButton(QString("Shine"));
	m_pBackRadioButton[5] = new QRadioButton(QString("Atmosphere"));
	m_pBackRadioButton[6] = new QRadioButton(QString("Snow"));
	
	m_pBackRadioButton[iBack]->setChecked(true);
	
	for(unsigned int i = 0; i < 7; ++i)
		m_pVBackLayoutLeft->addWidget(m_pBackRadioButton[i]);
	
	for(unsigned int i = 0; i < 7; ++i)
		connect(m_pBackRadioButton[i], SIGNAL(toggled(bool)), this, SLOT(sltBack(bool)));
		
	m_pVBackLayoutRight->addWidget(m_pBackPreview);
	m_pVBackLayoutRight->addWidget(m_pLabelBackImage);
	
	m_pVBackLayoutRight->setAlignment(Qt::AlignTop);
	
	m_pHBackLayout->addLayout(m_pVBackLayoutLeft);
	m_pHBackLayout->addLayout(m_pVBackLayoutRight);
	
	m_pBackLayout->addLayout(m_pHBackLayout);
}

void BKOptions::createKeyTab()
{
	m_pKeyTab = new QWidget();
	m_pKeyLayout = new QGridLayout();
	
	m_pLabelKey1 = new QLabel(tr("Carta sinistra"));
	m_pLabelKey2 = new QLabel(tr("Carta centrale"));
	m_pLabelKey3 = new QLabel(tr("Carta destra"));
	
	m_pComboBoxKey[0] = new QComboBox();
	m_pComboBoxKey[1] = new QComboBox();
	m_pComboBoxKey[2] = new QComboBox();
	
	setComboBox(m_pComboBoxKey[0]);
	setComboBox(m_pComboBoxKey[1]);
	setComboBox(m_pComboBoxKey[2]);
	
	m_pKeyLayout->addWidget(m_pLabelKey1, 1, 1, Qt::AlignLeft);
	m_pKeyLayout->addWidget(m_pComboBoxKey[0], 1, 2, Qt::AlignLeft);
	m_pKeyLayout->addWidget(m_pLabelKey2, 2, 1, Qt::AlignLeft);
	m_pKeyLayout->addWidget(m_pComboBoxKey[1], 2, 2, Qt::AlignLeft);
	m_pKeyLayout->addWidget(m_pLabelKey3, 3, 1, Qt::AlignLeft);
	m_pKeyLayout->addWidget(m_pComboBoxKey[2], 3, 2, Qt::AlignLeft);
	
	m_pKeyTab->setLayout(m_pKeyLayout);
}

void BKOptions::createSpeedTab()
{
	m_pSpeedTab = new QWidget();
	m_pSpeedLayout = new QVBoxLayout();
	m_pSpeedFooter = new QHBoxLayout();
	
	m_pRadioSpeedHigh = new QRadioButton(tr("Alta (500 ms)"));
	m_pRadioSpeedNormal = new QRadioButton(tr("Media (1000 ms)"));
	m_pRadioSpeedSlow = new QRadioButton(tr("Bassa (2000 ms)"));
	m_pRadioSpeedManual = new QRadioButton(tr("Manuale (100 ms - 5000 ms)"));
	
	m_pCheckDelayCPU = new QCheckBox(tr("Includi un ritardo quando gioca il computer "));
	
	m_pSpinBoxManualSpeed = new QSpinBox();
	m_pSpinBoxManualSpeed->setMinimum(100);
	m_pSpinBoxManualSpeed->setMaximum(5000);
	m_pSpinBoxManualSpeed->setValue(1000);
	m_pSpinBoxManualSpeed->setFixedWidth(60);
	m_pSpinBoxManualSpeed->setAlignment(Qt::AlignRight);
	
	//m_pSpinBoxManualSpeed->setReadOnly(true);
	m_pSpinBoxManualSpeed->setEnabled(false);
	
	m_pSpeedFooter->addWidget(m_pSpinBoxManualSpeed);
	m_pSpeedFooter->setAlignment(Qt::AlignRight);
	
	m_pSliderSpeed = new QSlider();
	m_pSliderSpeed->setOrientation(Qt::Horizontal);
	m_pSliderSpeed->setRange(100, 5000);
	m_pSliderSpeed->setSliderPosition(1000);
	m_pSliderSpeed->setEnabled(false);
	
	m_pSpeedLayout->addWidget(m_pRadioSpeedHigh);
	m_pSpeedLayout->addWidget(m_pRadioSpeedNormal);
	m_pSpeedLayout->addWidget(m_pRadioSpeedSlow);
	m_pSpeedLayout->addWidget(m_pRadioSpeedManual);
	m_pSpeedLayout->addWidget(m_pSliderSpeed);
	m_pSpeedLayout->addLayout(m_pSpeedFooter);
	m_pSpeedLayout->addWidget(m_pCheckDelayCPU);
	
	m_pSpeedTab->setLayout(m_pSpeedLayout);
	
	connect(m_pRadioSpeedManual, SIGNAL(toggled(bool)), this, SLOT(sltManualSpeed()));
	connect(m_pSliderSpeed, SIGNAL(valueChanged(int)), this, SLOT(sltSlider()));
	connect(m_pSpinBoxManualSpeed, SIGNAL(valueChanged(int)), this, SLOT(sltSpinBox()));
}

void BKOptions::createGeneralTab()
{
	m_pGeneralTab = new QWidget();
	
	m_pGeneralLayout = new QVBoxLayout();
	
	m_pRadioGeneral[0] = new QRadioButton(tr("Inizia il giocatore"));
	m_pRadioGeneral[1] = new QRadioButton(tr("Inizia il computer"));
	m_pRadioGeneral[2] = new QRadioButton(tr("Scelta casuale del giocatore iniziale"));
	
	m_pCheckBoxGeneralPoints = new QCheckBox(tr("Mostra punteggio"));
	m_pCheckBoxGeneralSquare = new QCheckBox(tr("Mostra riquadro intorno alle carte"));
	
	m_pGeneralLayout->addWidget(m_pRadioGeneral[0]);
	m_pGeneralLayout->addWidget(m_pRadioGeneral[1]);
	m_pGeneralLayout->addWidget(m_pRadioGeneral[2]);
	m_pGeneralLayout->addWidget(m_pCheckBoxGeneralPoints);
	m_pGeneralLayout->addWidget(m_pCheckBoxGeneralSquare);
	
	m_pGeneralTab->setLayout(m_pGeneralLayout);
}

void BKOptions::setComboBox(QComboBox * p_pComboBox)
{
	for(unsigned int i = 0; i < 26; ++i)
	{
		char c = i + 'A';
		
		p_pComboBox->addItem(QString(QChar(c)));
	}
	
	for(unsigned int i = 0; i < 10; ++i)
	{
		char c = i + '0';
		
		p_pComboBox->addItem(QString(QChar(c)));
	}
	
	p_pComboBox->addItem(QString("Spazio"));
	p_pComboBox->addItem(QString("Freccia su"));
	p_pComboBox->addItem(QString(trUtf8("Freccia giù")));
	p_pComboBox->addItem(QString("Freccia destra"));
	p_pComboBox->addItem(QString("Freccia sinistra"));
	
	p_pComboBox->setMaxVisibleItems(10);
}

void BKOptions::setCurrentOptionsChecked()
{
	// Setting deck using configuration parameter. Set this after
	// the connect function due to trig the related event.
	m_pDeckRadioButton[BKConfig::getDeck(1)]->setChecked(true);
	
	int iBack = BKConfig::getBackground(0);
	
	m_pBackRadioButton[iBack]->setChecked(true);
	
	// Setting configuration parameters
	int iSpeed = BKConfig::getTimeout(1000);
	
	switch(iSpeed)
	{
		case 500:
			m_pRadioSpeedHigh->setChecked(true);
		break;
		case 1000:
			m_pRadioSpeedNormal->setChecked(true);
		break;
		case 2000:
			m_pRadioSpeedSlow->setChecked(true);
		break;
		default:
			m_pRadioSpeedManual->setChecked(true);
			m_pSliderSpeed->setValue(iSpeed);
		break;
	}
	
	m_pRadioGeneral[BKConfig::getPlayerThatStarts(2)]->setChecked(true);
	m_pCheckBoxGeneralPoints->setChecked(BKConfig::getVisiblePoints(true));
	m_pCheckBoxGeneralSquare->setChecked(BKConfig::getSquareAroundCards(true));
	
	m_pComboBoxKey[0]->setCurrentIndex(BKConfig::getKeyId("KeyId_0", 27));
	m_pComboBoxKey[1]->setCurrentIndex(BKConfig::getKeyId("KeyId_1", 28));
	m_pComboBoxKey[2]->setCurrentIndex(BKConfig::getKeyId("KeyId_2", 29));
	
	m_pCheckDelayCPU->setChecked(BKConfig::getDelayCPU(false));
}

void BKOptions::sltDeck(bool p_bValue)
{
	if (!p_bValue) return;
	
	for(unsigned int i = 0; i < 7; ++i)
	{
		if(m_pDeckRadioButton[i]->isChecked())
		{
			char c = i + '0';
			m_pDeckImage->load(_STR_DIR_IMG_DECKS + QString("/bkImgDeck_0") + QString(QChar(c)) + QString(".png"));
			break;
		}
	}
	
	
	
	m_pLabelImage->setPixmap(* m_pDeckImage);
}

void BKOptions::sltBack(bool p_bValue)
{
	if (!p_bValue) return;
	
	unsigned int i;
	
	for(i = 0; i < 7; ++i)
	{
		if(m_pBackRadioButton[i]->isChecked())
		{
			break;
		}
	}
	
	char c = i + '0';
	m_pBackImage->load(_STR_DIR_IMG_BACKGROUND + QString("/bkImgBackground_0") + c + "_small.png");
	m_pLabelBackImage->setPixmap(*m_pBackImage);
}

void BKOptions::sltButtonOk()
{
	int indexCombo[3], timeOut = 1000;
	
	indexCombo[0] = m_pComboBoxKey[0]->currentIndex();
	indexCombo[1] = m_pComboBoxKey[1]->currentIndex();
	indexCombo[2] = m_pComboBoxKey[2]->currentIndex();
	
	if(indexCombo[0] == indexCombo[1] || indexCombo[0] == indexCombo[2] || indexCombo[1] == indexCombo[2])
	{
		if(!m_pWindowAlert)
			m_pWindowAlert = new BKAlert(this);
		m_pWindowAlert->show();
	}
	else
	{
		for(unsigned int i = 0; i < 7; ++i)
		{
			if(m_pDeckRadioButton[i]->isChecked())
			{
				BKConfig::setDeck(i);
				break;
			}
		}
	
		for(unsigned int i = 0; i < 7; ++i)
		{
			if(m_pBackRadioButton[i]->isChecked())
			{
				BKConfig::setBackground(i);
				break;
			}
		}
		
		for(unsigned int i = 0; i < 3; ++i)
		{
			if(m_pRadioGeneral[i]->isChecked())
			{
				BKConfig::setPlayerThatStarts(i);
				break;
			}
		}
		
		BKConfig::setKey("Key_0", "KeyId_0", indexCombo[0]);
		BKConfig::setKey("Key_1", "KeyId_1", indexCombo[1]);
		BKConfig::setKey("Key_2", "KeyId_2", indexCombo[2]);
		
		if(m_pRadioSpeedHigh->isChecked())
			timeOut = 500;
		else if(m_pRadioSpeedNormal->isChecked())
			timeOut = 1000;
		else if(m_pRadioSpeedSlow->isChecked())
			timeOut = 2000;
		else if(m_pRadioSpeedManual->isChecked())
			timeOut = m_pSliderSpeed->sliderPosition();
		
		BKConfig::setTimeout(timeOut);
		
		BKConfig::setVisiblePoints(m_pCheckBoxGeneralPoints->isChecked());
		BKConfig::setSquareAroundCards(m_pCheckBoxGeneralSquare->isChecked());
		
		BKConfig::setDelayCPU(m_pCheckDelayCPU->isChecked());
		
		hide();
	}
}

void BKOptions::sltButtonReset()
{
	hide();
}

void BKOptions::sltManualSpeed()
{
	if(m_pRadioSpeedManual->isChecked())
	{
		m_pSliderSpeed->setEnabled(true);
		m_pSpinBoxManualSpeed->setEnabled(true);
	}
	else
	{
		m_pSliderSpeed->setEnabled(false);
		m_pSpinBoxManualSpeed->setEnabled(false);
	}
}

void BKOptions::sltSlider()
{
	/*QString str;
	str.setNum(m_pSliderSpeed->sliderPosition());
	m_pSpinBoxManualSpeed->setText(str + QString(" ms"));*/
	m_pSpinBoxManualSpeed->setValue(m_pSliderSpeed->sliderPosition());
	//m_pSpinBoxManualSpeed->setAlignment(Qt::AlignRight);
}

void BKOptions::sltSpinBox()
{
	m_pSliderSpeed->setSliderPosition(m_pSpinBoxManualSpeed->value());
}

