//=============================================================================
//
//   File : BKGameBoard.cpp
//   Created on mer 07 ott 2009 12:21:42 by Iacopo Palazzi
//
//   This file is part of the BrisKola distribution
//   Copyright (C) 2009 Iacopo Palazzi <iacopo.palazzi@gmail.com>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================

#include "BKGameBoard.h"

#include "BK.h"
#include "BKCard.h"
#include "BKConfig.h"
#include "BKGameView.h"
#include "BKGameScene.h"
#include "BKMatch.h"
#include "BKController.h"

#include <QPoint>
#include <QGraphicsTextItem>
#include <QGraphicsPixmapItem>
#include <QTimer>

// The one and only BKGameBoardobject
BKGameBoard * BKGameBoard::m_pInstance = NULL;

BKGameBoard::BKGameBoard(QWidget * p_pParent)
: QFrame(p_pParent)
{
	create();
	connect(BKConfig::instance(), SIGNAL(sgnDeckChanged()), this, SLOT(doSetDeckImage()));
	connect(BKConfig::instance(), SIGNAL(sgnVisiblePointsChanged(bool)), this, SLOT(doSetVisiblePoints(bool)));
	connect(BKConfig::instance(), SIGNAL(sgnSquareAroundCardsChanged(bool)), this, SLOT(doSetSquareAroundCards(bool)));
	connect(BKConfig::instance(), SIGNAL(sgnSquareAroundCardsChanged(bool)), this, SLOT(doSetSquareAroundCards(bool)));
}

BKGameBoard::~BKGameBoard()
{
	if (m_pBlinker) delete m_pBlinker;

	if (m_pPlacesCPUSelector) delete m_pPlacesCPUSelector;
	if (m_pPlacesHUMSelector) delete m_pPlacesHUMSelector;

	if (m_pPlacesHUM) delete [] m_pPlacesHUM;
	if (m_pPlacesCPU) delete [] m_pPlacesCPU;
	if (m_rPlacePlayedCard) delete [] m_rPlacePlayedCard;
	
	if (m_pQSDeck) delete m_pQSDeck;
	if (m_pQSGhostBrisKola) delete m_pQSGhostBrisKola;
	if (m_pQSGhostCPUPlayed) delete m_pQSGhostCPUPlayed;
	if (m_pQSGhostHUMPlayed) delete m_pQSGhostHUMPlayed;
	if (m_pArrowTurn) delete m_pArrowTurn;
	
	if (m_pWin) delete m_pWin;
	if (m_pLose) delete m_pLose;
	
	if (m_pLblCPU) delete m_pLblCPU;
	if (m_pLblHUM) delete m_pLblHUM;
	if (m_pGPixmapHUMUnitPoints) delete m_pGPixmapHUMUnitPoints;
	if (m_pGPixmapHUMDecPoints) delete m_pGPixmapHUMDecPoints;
	if (m_pGPixmapCPUUnitPoints) delete m_pGPixmapCPUUnitPoints;
	if (m_pGPixmapCPUDecPoints) delete m_pGPixmapCPUDecPoints;
	if (m_pGPixmapWinner) delete m_pGPixmapWinner;
	if (m_pPixmapHUMUnitPoints) delete m_pPixmapHUMUnitPoints;
	if (m_pPixmapHUMDecPoints) delete m_pPixmapHUMDecPoints;
	if (m_pPixmapCPUUnitPoints) delete m_pPixmapCPUUnitPoints;
	if (m_pPixmapCPUDecPoints) delete m_pPixmapCPUDecPoints;
	if (m_pGPixmapHand) delete m_pGPixmapHand;
	if (m_pGPixmapSmallUnits) delete m_pGPixmapSmallUnits;
	if (m_pGPixmapSmallDec) delete m_pGPixmapSmallDec;
	if (m_pPixmapHand) delete m_pPixmapHand;
	if (m_pPixmapSmallUnits) delete m_pPixmapSmallUnits;
	if (m_pPixmapSmallDec) delete m_pPixmapSmallDec;
	if (m_pPixmapWinner) delete m_pPixmapWinner;
	// if (m_pTxtHands) delete m_pTxtHands;
	
	BKGameView::done();
	BKGameScene::done();
}

bool BKGameBoard::init(QWidget * p_pParent)
{
	if(m_pInstance)return false;
	m_pInstance = new BKGameBoard(p_pParent);
	return true;
}

void BKGameBoard::done()
{
	if(!m_pInstance)return;
	delete m_pInstance;
	m_pInstance = NULL;
}

void BKGameBoard::doSetDeckImage()
{
	m_pInstance->m_pQSDeck->setPixmap(BKConfig::getImage(BKConfig::IMG_DECK));
}

void BKGameBoard::doSetVisiblePoints(bool p_bVisiblePoints)
{
	m_pInstance->m_pGPixmapCPUUnitPoints->setVisible(p_bVisiblePoints);
	if(BKController::match()->getCPUPoints() > 9)
		m_pInstance->m_pGPixmapCPUDecPoints->setVisible(p_bVisiblePoints);
	
	m_pInstance->m_pGPixmapHUMUnitPoints->setVisible(p_bVisiblePoints);
	if(BKController::match()->getHUMPoints() > 9)
		m_pInstance->m_pGPixmapHUMDecPoints->setVisible(p_bVisiblePoints);
}

void BKGameBoard::doSetSquareAroundCards(bool p_bSquareAroundCards)
{
	BKMatch::eMatchState matchState = BKController::match()->matchState();
	
	if(matchState == BKMatch::MS_WAITING_HUM_CARD || matchState == BKMatch::MS_WAITING_CPU_CARD)
		m_pInstance->m_pArrowTurn->setVisible(p_bSquareAroundCards);
}

void BKGameBoard::doSetWinner(unsigned int p_uiPlayerPoints)
{
	if(p_uiPlayerPoints > 60)
	{
		m_pPixmapWinner->load(_STR_DIR_IMG_TEXT + QString("/bkText_win.png"));
		m_pGPixmapWinner->setPixmap(* m_pPixmapWinner);
	}
	else if(p_uiPlayerPoints < 60)
	{
		m_pPixmapWinner->load(_STR_DIR_IMG_TEXT + QString("/bkText_lose.png"));
		m_pGPixmapWinner->setPixmap(* m_pPixmapWinner);
	}
	else
	{
		m_pPixmapWinner->load(_STR_DIR_IMG_TEXT + QString("/bkText_even.png"));
		m_pGPixmapWinner->setPixmap(* m_pPixmapWinner);
	}
	
	m_pGPixmapWinner->setVisible(true);
}

void BKGameBoard::hideTxtWinner()
{
	m_pGPixmapWinner->setVisible(false);
}

// ================================
// Start putting your stuff here...
// ================================

void BKGameBoard::create()
{
	BKGameScene::init(this);
	
	BKGameView::init(BKGameScene::instance(), this);
	BKGameView::instance()->setRenderHints( QPainter::Antialiasing );
	BKGameView::instance()->show();
	
	m_pPlacesCPU = new QPoint[3];
		m_pPlacesCPU[0] = QPoint(155, 40);
		m_pPlacesCPU[1] = QPoint(242, 40);
		m_pPlacesCPU[2] = QPoint(329, 40);
		
	m_pPlacesHUM = new QPoint[3];
		m_pPlacesHUM[0] = QPoint(155, 290);
		m_pPlacesHUM[1] = QPoint(242, 290);
		m_pPlacesHUM[2] = QPoint(329, 290);
	
	m_pPlacesCPUSelector = new QPoint(140,  25);
	m_pPlacesHUMSelector = new QPoint(140, 275);

	m_rPlacePlayedCard = new QPoint[2];
		m_rPlacePlayedCard[0] = QPoint(198, 165);
		m_rPlacePlayedCard[1] = QPoint(285, 165);
	
	m_rPlaceDeck = QPoint(30, 166);
	m_rPlaceBrisKola = QPoint(60, 126);
	
	m_pQSDeck = new BKGameSceneItem();
		m_pQSDeck->setPixmap(BKConfig::getImage(BKConfig::IMG_DECK));
		m_pQSDeck->setVisible(false);
		m_pQSDeck->setPos(m_rPlaceDeck);
	
	m_pQSGhostBrisKola = new BKGameSceneItem();
		m_pQSGhostBrisKola->setVisible(false);
		m_pQSGhostBrisKola->setTransparency(0.5);
		m_pQSGhostBrisKola->setPos(m_rPlaceBrisKola);
	
	m_pQSGhostCPUPlayed = new BKGameSceneItem();
		m_pQSGhostCPUPlayed->setVisible(false);
		m_pQSGhostCPUPlayed->setTransparency(0.25);
		
	m_pQSGhostHUMPlayed = new BKGameSceneItem();
		m_pQSGhostHUMPlayed->setVisible(false);
		m_pQSGhostHUMPlayed->setTransparency(0.25);
	
	m_pArrowTurn = new BKGameSceneItem();
		m_pArrowTurn->setPixmap(BKConfig::getImage(BKConfig::IMG_MISC_TURN));
		m_pArrowTurn->setVisible(false);
		m_pArrowTurn->setZValue(0);
	
	m_pWin = new BKGameSceneItem();
		m_pWin->setPixmap(BKConfig::getImage(BKConfig::IMG_MISC_WIN));
		m_pWin->setVisible(false);
		
	m_pLose = new BKGameSceneItem();
		m_pLose->setPixmap(BKConfig::getImage(BKConfig::IMG_MISC_LOSE));
		m_pLose->setVisible(false);
		
	m_pPixmapHUMUnitPoints = new QPixmap();
	m_pPixmapHUMDecPoints = new QPixmap();
	m_pPixmapCPUUnitPoints = new QPixmap();
	m_pPixmapCPUDecPoints = new QPixmap();
	m_pPixmapWinner = new QPixmap();
	
	m_pPixmapHand = new QPixmap(_STR_DIR_IMG_TEXT + QString("/bkText_hand.png"));
	m_pPixmapSmallUnits = new QPixmap(_STR_DIR_IMG_TEXT + QString());
	m_pPixmapSmallDec = new QPixmap(_STR_DIR_IMG_TEXT + QString());
	
	m_pGPixmapHUMUnitPoints = new QGraphicsPixmapItem(* m_pPixmapHUMUnitPoints);
		m_pGPixmapHUMUnitPoints->setVisible(BKConfig::instance()->getVisiblePoints(true));
		m_pGPixmapHUMUnitPoints->setPos(560, 330);
	m_pGPixmapHUMDecPoints = new QGraphicsPixmapItem(* m_pPixmapHUMDecPoints);
		m_pGPixmapHUMDecPoints->setVisible(false);
		m_pGPixmapHUMDecPoints->setPos(540, 330);
	
	m_pGPixmapCPUUnitPoints = new QGraphicsPixmapItem(* m_pPixmapCPUUnitPoints);
		m_pGPixmapCPUUnitPoints->setVisible(BKConfig::instance()->getVisiblePoints(true));
		m_pGPixmapCPUUnitPoints->setPos(560, 76);
	m_pGPixmapCPUDecPoints = new QGraphicsPixmapItem(* m_pPixmapCPUDecPoints);
		m_pGPixmapCPUDecPoints->setVisible(false);
		m_pGPixmapCPUDecPoints->setPos(540, 76);
	
	m_pGPixmapWinner = new QGraphicsPixmapItem(* m_pPixmapWinner);
		m_pGPixmapWinner->setVisible(false);
		m_pGPixmapWinner->setPos(480, 190);
	
	m_pLblCPU = new QGraphicsTextItem();
	m_pLblHUM = new QGraphicsTextItem();
	
	m_pGPixmapHand = new QGraphicsPixmapItem(* m_pPixmapHand);
		m_pGPixmapHand->setVisible(false);
		m_pGPixmapHand->setPos(30, 280);
	
	m_pGPixmapSmallUnits = new QGraphicsPixmapItem(* m_pPixmapSmallUnits);
		m_pGPixmapSmallUnits->setVisible(false);
		m_pGPixmapSmallUnits->setPos(95, 280);
	
	m_pGPixmapSmallDec = new QGraphicsPixmapItem(* m_pPixmapSmallDec);
		m_pGPixmapSmallDec->setVisible(false);
		m_pGPixmapSmallDec->setPos(85, 280);
	
	/* m_pTxtHands = new QGraphicsTextItem();
		m_pTxtHands->setDefaultTextColor(QColor(255, 255, 255, 255));
		m_pTxtHands->setVisible(true);
		m_pTxtHands->setPos(30, 280);
		m_pTxtHands->setHtml("-----"); */
	
	m_iBlinkState = 0;
	m_pBlinker = new QTimer();
	connect(m_pBlinker, SIGNAL(timeout()), this, SLOT(doBlink()));

	BKGameScene::instance()->addItem(m_pQSDeck);
	BKGameScene::instance()->addItem(m_pQSGhostBrisKola);
	BKGameScene::instance()->addItem(m_pQSGhostCPUPlayed);
	BKGameScene::instance()->addItem(m_pQSGhostHUMPlayed);
	BKGameScene::instance()->addItem(m_pArrowTurn);
	BKGameScene::instance()->addItem(m_pWin);
	BKGameScene::instance()->addItem(m_pLose);
	
	BKGameScene::instance()->addItem(m_pGPixmapHUMUnitPoints);
	BKGameScene::instance()->addItem(m_pGPixmapHUMDecPoints);
	BKGameScene::instance()->addItem(m_pGPixmapCPUUnitPoints);
	BKGameScene::instance()->addItem(m_pGPixmapCPUDecPoints);
	BKGameScene::instance()->addItem(m_pGPixmapWinner);
	
	BKGameScene::instance()->addItem(m_pGPixmapHand);
	BKGameScene::instance()->addItem(m_pGPixmapSmallUnits);
	BKGameScene::instance()->addItem(m_pGPixmapSmallDec);
	// BKGameScene::instance()->addItem(m_pTxtHands);
}

void BKGameBoard::reset()
{
	BKGameScene::reset();
	m_pInstance->m_pQSDeck->setVisible(false);
	m_pInstance->m_pQSGhostBrisKola->setVisible(false);
	
	m_pInstance->m_pPixmapHUMUnitPoints->load(_STR_DIR_IMG_TEXT + QString("/bkNumber_0.png"));
	m_pInstance->m_pGPixmapHUMUnitPoints->setPixmap(* m_pInstance->m_pPixmapHUMUnitPoints);
	
	m_pInstance->m_pPixmapHUMDecPoints->load(_STR_DIR_IMG_TEXT + QString("/bkNumber_0.png"));
	m_pInstance->m_pGPixmapHUMDecPoints->setPixmap(* m_pInstance->m_pPixmapHUMDecPoints);
	m_pInstance->m_pGPixmapHUMDecPoints->setVisible(false);
	
	m_pInstance->m_pPixmapCPUUnitPoints->load(_STR_DIR_IMG_TEXT + QString("/bkNumber_0.png"));
	m_pInstance->m_pGPixmapCPUUnitPoints->setPixmap(* m_pInstance->m_pPixmapCPUUnitPoints);
	
	m_pInstance->m_pPixmapCPUDecPoints->load(_STR_DIR_IMG_TEXT + QString("/bkNumber_0.png"));
	m_pInstance->m_pGPixmapCPUDecPoints->setPixmap(* m_pInstance->m_pPixmapCPUDecPoints);
	m_pInstance->m_pGPixmapCPUDecPoints->setVisible(false);
}

void BKGameBoard::placeBrisKola(BKCard * p_pCard)
{
	m_pInstance->m_pQSGhostBrisKola->setVisible(true);
	m_pInstance->m_pQSGhostBrisKola->setPixmap(((QGraphicsPixmapItem *)p_pCard->card())->pixmap());
	m_pInstance->m_pQSGhostBrisKola->setZValue(m_pInstance->m_pQSDeck->zValue() - 1);
	m_pInstance->m_pQSGhostBrisKola->setTransparency(1.0);
}

void BKGameBoard::placePlayedCard(unsigned int p_uiPos, BKCard * p_pCard)
{
	dbg("BKGameBoard::placePlayedCard(unsigned int p_uiPos, BKCard * p_pCard)");
	
	BKGameSceneItem * ghost;
	
	if (p_pCard->player()->type() == BKPlayer::PT_HUM)
		ghost = m_pInstance->m_pQSGhostHUMPlayed;
	else
		ghost = m_pInstance->m_pQSGhostCPUPlayed;

	ghost->setPixmap(*p_pCard->card()->pixmapVisible());
	ghost->setPos(p_pCard->card()->pos());
	ghost->setVisible(true);

	p_pCard->setHidden(false);
	placeCard(p_pCard, m_pInstance->m_rPlacePlayedCard[p_uiPos]);
	
}

void BKGameBoard::placeCPUCard(unsigned int p_uiPos, BKCard * p_pCard)
{
	placeCard(p_pCard, m_pInstance->m_pPlacesCPU[p_uiPos]);
}

void BKGameBoard::placeHUMCard(unsigned int p_uiPos, BKCard * p_pCard)
{
	placeCard(p_pCard, m_pInstance->m_pPlacesHUM[p_uiPos]);
}

void BKGameBoard::placeCard(BKCard * p_pCard, QPoint & p_pPoint)
{
	p_pCard->card()->show();
	p_pCard->card()->setPos(p_pPoint);
	if (!p_pCard->card()->scene())
		BKGameScene::instance()->addItem((QGraphicsItem *)p_pCard->card());
}

void BKGameBoard::setCPUTurn()
{
	instance()->m_iBlinkState = 0;
	instance()->m_pArrowTurn->setVisible(false);
	//m_pInstance->m_pArrowTurn->setPos(440, 66);
	instance()->m_pArrowTurn->setPos(*(instance()->m_pPlacesCPUSelector));
}

void BKGameBoard::setHUMTurn()
{
	instance()->m_iBlinkState = 0;
	if(BKConfig::getSquareAroundCards(true)) instance()->m_pArrowTurn->setVisible(true);
	//m_pInstance->m_pArrowTurn->setPos(440, 320);
	instance()->m_pArrowTurn->setPos(*(instance()->m_pPlacesHUMSelector));
}

void BKGameBoard::startBlinkingWinner(BKPlayer::eType p_eWinner)
{
	switch(p_eWinner)
	{
		case BKPlayer::PT_HUM:
			instance()->m_rBlinkingPlace = *(instance()->m_pPlacesHUMSelector);
		break;
		case BKPlayer::PT_CPU:
			instance()->m_rBlinkingPlace = *(instance()->m_pPlacesCPUSelector);
		break;
	}

	instance()->m_pArrowTurn->setPos(instance()->m_rBlinkingPlace );
	instance()->m_iBlinkState = 1;
	instance()->doBlink();
}

void BKGameBoard::setWin()
{
	m_pInstance->m_pWin->setVisible(true);
	m_pInstance->m_pWin->setPos(432, 182);
}

void BKGameBoard::setLose()
{
	m_pInstance->m_pArrowTurn->setVisible(false);
	m_pInstance->m_pLose->setVisible(true);
	m_pInstance->m_pLose->setPos(432, 182);
}

void BKGameBoard::resetIcons()
{
	m_pInstance->m_pArrowTurn->setVisible(false);
	m_pInstance->m_pLose->setVisible(false);
	m_pInstance->m_pWin->setVisible(false);
	m_pInstance->m_pQSGhostHUMPlayed->setVisible(false);
	m_pInstance->m_pQSGhostCPUPlayed->setVisible(false);
}

void BKGameBoard::setTextHands(unsigned int p_uiValue)
{
	// m_pInstance->m_pTxtHands->setHtml(QString("Mano <b>%1<b>").arg(p_uiValue));
	char cUnits, cDec;
	
	cUnits = (p_uiValue % 10) + '0';
	cDec = (p_uiValue/10) + '0';
	
	if(p_uiValue < 10)
	{
		m_pInstance->m_pPixmapSmallDec->load(_STR_DIR_IMG_TEXT + QString("/bkNumber_") + QString(QChar(cUnits)) + QString("_small.png"));
		m_pInstance->m_pGPixmapSmallDec->setPixmap(* m_pInstance->m_pPixmapSmallDec);
	}
	else
	{
		m_pInstance->m_pPixmapSmallUnits->load(_STR_DIR_IMG_TEXT + QString("/bkNumber_") + QString(QChar(cUnits)) + QString("_small.png"));
		m_pInstance->m_pGPixmapSmallUnits->setPixmap(* m_pInstance->m_pPixmapSmallUnits);
		m_pInstance->m_pGPixmapSmallUnits->setVisible(true);
		
		m_pInstance->m_pPixmapSmallDec->load(_STR_DIR_IMG_TEXT + QString("/bkNumber_") + QString(QChar(cDec)) + QString("_small.png"));
		m_pInstance->m_pGPixmapSmallDec->setPixmap(* m_pInstance->m_pPixmapSmallDec);
	}
	
	if(p_uiValue == 17)
	{ // Show "Ultima pescata"
		m_pInstance->m_pPixmapWinner->load(_STR_DIR_IMG_TEXT + QString("/bkText_last.png"));
		m_pInstance->m_pGPixmapWinner->setPixmap(* m_pInstance->m_pPixmapWinner);
		m_pInstance->m_pGPixmapWinner->setPos(461, 190);
		m_pInstance->m_pGPixmapWinner->setVisible(true);
	}
	else if(p_uiValue == 18)
	{ // Hide "Ultima pescata"
		m_pInstance->m_pGPixmapWinner->setVisible(false);
		m_pInstance->m_pGPixmapWinner->setPos(480, 190);
	}
}

void BKGameBoard::setTextCPUPoints(unsigned int p_uiValue)
{
	char cUnits, cDec;
	
	cDec = (p_uiValue/10) + '0';
	cUnits = (p_uiValue % 10) + '0';
	
	m_pInstance->m_pPixmapCPUUnitPoints->load(_STR_DIR_IMG_TEXT + QString("/bkNumber_") + QString(QChar(cUnits)) +  QString(".png"));
	m_pInstance->m_pGPixmapCPUUnitPoints->setPixmap(* m_pInstance->m_pPixmapCPUUnitPoints);
	
	m_pInstance->m_pPixmapCPUDecPoints->load(_STR_DIR_IMG_TEXT + QString("/bkNumber_") + QString(QChar(cDec)) + QString(".png"));
	m_pInstance->m_pGPixmapCPUDecPoints->setPixmap(* m_pInstance->m_pPixmapCPUDecPoints);
	if(cDec > '0')
		m_pInstance->m_pGPixmapCPUDecPoints->setVisible(BKConfig::instance()->getVisiblePoints(true));
}

void BKGameBoard::setTextHUMPoints(unsigned int p_uiValue)
{
	char cUnits, cDec;
	
	cDec = (p_uiValue/10) + '0';
	cUnits = (p_uiValue % 10) + '0';
	
	m_pInstance->m_pPixmapHUMUnitPoints->load(_STR_DIR_IMG_TEXT + QString("/bkNumber_") + QString(QChar(cUnits)) +  QString(".png"));
	m_pInstance->m_pGPixmapHUMUnitPoints->setPixmap(* m_pInstance->m_pPixmapHUMUnitPoints);
	
	m_pInstance->m_pPixmapHUMDecPoints->load(_STR_DIR_IMG_TEXT + QString("/bkNumber_") + QString(QChar(cDec)) + QString(".png"));
	m_pInstance->m_pGPixmapHUMDecPoints->setPixmap(* m_pInstance->m_pPixmapHUMDecPoints);
	if(cDec > '0')
		m_pInstance->m_pGPixmapHUMDecPoints->setVisible(BKConfig::instance()->getVisiblePoints(true));
}

void BKGameBoard::removeCard(BKCard * p_pCard)
{
	p_pCard->card()->hide();
}

void BKGameBoard::showDeck(bool p_bValue)
{
	m_pInstance->m_pQSDeck->setVisible(p_bValue);
}

void BKGameBoard::showGhostBrisKola(bool p_bValue)
{
	m_pInstance->m_pQSGhostBrisKola->setTransparency(!p_bValue ? 1.0 : 0.5);
}

void BKGameBoard::showHand()
{
	m_pInstance->m_pGPixmapHand->setVisible(true);
	m_pInstance->m_pGPixmapSmallDec->setVisible(true);
	m_pInstance->m_pGPixmapSmallUnits->setVisible(false);
}

void BKGameBoard::doBlink()
{
	switch(m_iBlinkState)
	{
		case 0: // Do not blink
			return;
		break;
		case 1: // Blink on
			m_iBlinkState = 2;
			if(BKConfig::getSquareAroundCards(true)) m_pInstance->m_pArrowTurn->setVisible(true);
		break;
		case 2: // Blink on
			m_iBlinkState = 1;
			m_pInstance->m_pArrowTurn->setVisible(false);
		break;
	}
	instance()->m_pBlinker->start(100);
}

