#ifndef _BKGAMEBOARD_H_
#define _BKGAMEBOARD_H_
//=============================================================================
//
//   File : BKGameBoard.h
//   Created on mer 07 ott 2009 12:21:42 by Iacopo Palazzi
//
//   This file is part of the BrisKola distribution
//   Copyright (C) 2009 Iacopo Palazzi <iacopo.palazzi@gmail.com>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================

#include <QFrame>

#include <QPoint>

#include "BKPlayer.h"

class BKCard;
class BKGameSceneItem;
class QWidget;
class QGraphicsTextItem;
class QString;
class QTimer;
class QPixmap;
class QGraphicsPixmapItem;

class BKGameBoard : public QFrame
{
	Q_OBJECT
protected:
	BKGameBoard(QWidget * p_pParent);
	~BKGameBoard();

// ================================
// Put members declaration below...
// ================================
private:
	QPoint * m_pPlacesCPU;
	QPoint * m_pPlacesHUM;
	QPoint * m_rPlacePlayedCard;
	QPoint * m_pPlacesCPUSelector;
	QPoint * m_pPlacesHUMSelector;

	QPoint m_rPlaceDeck;
	QPoint m_rPlaceBrisKola;
	QPoint m_rBlinkingPlace;

	BKGameSceneItem * m_pQSDeck;
	BKGameSceneItem * m_pQSBrisKola;
	BKGameSceneItem * m_pQSGhostBrisKola;
	BKGameSceneItem * m_pQSGhostCPUPlayed;
	BKGameSceneItem * m_pQSGhostHUMPlayed;
	BKGameSceneItem * m_pArrowTurn;
	BKGameSceneItem * m_pWin;
	BKGameSceneItem * m_pLose;

	QGraphicsTextItem * m_pLblCPU;
	QGraphicsTextItem * m_pLblHUM;
	// QGraphicsTextItem * m_pTxtHands;
	
	QPixmap * m_pPixmapHUMUnitPoints, * m_pPixmapHUMDecPoints, * m_pPixmapCPUUnitPoints, * m_pPixmapCPUDecPoints, * m_pPixmapWinner;
	QPixmap * m_pPixmapHand, * m_pPixmapSmallUnits, * m_pPixmapSmallDec;
	QGraphicsPixmapItem * m_pGPixmapHUMUnitPoints, * m_pGPixmapHUMDecPoints, * m_pGPixmapCPUUnitPoints, * m_pGPixmapCPUDecPoints, * m_pGPixmapWinner;
	QGraphicsPixmapItem * m_pGPixmapHand, * m_pGPixmapSmallUnits, * m_pGPixmapSmallDec;

	QTimer * m_pBlinker;
	
	int m_iBlinkState; // 0: do not blink - 1: blink on - 2: blink off

protected:

public:


// ================================
// Put methods declaration below...
// ================================
private:
	void create();
	
	static void placeCard(BKCard * p_pCard, QPoint & p_pPoint);

protected:

public:
	static void placeCPUCard(unsigned int p_uiPos, BKCard * p_pCard);
	static void placeHUMCard(unsigned int p_uiPos, BKCard * p_pCard);
	static void placePlayedCard(unsigned int p_uiPos, BKCard * p_pCard);
	static void placeBrisKola(BKCard * p_pCard);

	static void removeCard(BKCard * p_pCard);

	static void showDeck(bool p_bValue);
	static void showGhostBrisKola(bool p_bValue);
	static void showHand();

	static void setCPUTurn();
	static void setHUMTurn();
	static void setWin();
	static void setLose();
	static void startBlinkingWinner(BKPlayer::eType p_eWinner);

	static void setTextCPUPoints(unsigned int p_uiValue);
	static void setTextHUMPoints(unsigned int p_uiValue);
	static void setTextHands(unsigned int p_uiValue);

	static void reset();
	static void resetIcons();

// Class follows singleton pattern
protected:
	static BKGameBoard * m_pInstance;
public:
	static BKGameBoard * instance() { return m_pInstance; };
	static bool init(QWidget * p_pParent);
	static void done();

public slots:
	void doBlink();
	void doSetDeckImage();
	void doSetVisiblePoints(bool p_bVisiblePoints);
	void doSetSquareAroundCards(bool p_bSquareAroundCards);
	void doSetWinner(unsigned int p_uiPlayerPoints);
	void hideTxtWinner();
};

#endif //!_BKGAMEBOARD_H_
