//=============================================================================
//
//   File : BKGameScene.cpp
//   Created on mar 06 ott 2009 16:11:54 by Iacopo Palazzi
//
//   This file is part of the BrisKola distribution
//   Copyright (C) 2009 Iacopo Palazzi <iacopo.palazzi@gmail.com>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================

#include "BKGameScene.h"

#include "BK.h"
#include "BKConfig.h"
#include "BKController.h"
#include "BKDeck.h"
#include "BKGameBoard.h"
#include "BKGameSceneItem.h"
#include "BKMatch.h"

#include <QWidget>

// The one and only BKGameSceneobject
BKGameScene * BKGameScene::m_pInstance = NULL;

BKGameScene::BKGameScene(QWidget * p_pParent)
: QGraphicsScene(p_pParent)
{
	createScene();
}

BKGameScene::~BKGameScene()
{
}

bool BKGameScene::init(QWidget * p_pParent)
{
	if(m_pInstance)return false;
	m_pInstance = new BKGameScene(p_pParent);
	return true;
}

void BKGameScene::done()
{
	if(!m_pInstance)return;
	delete m_pInstance;
	m_pInstance = NULL;
}


// ================================
// Start putting your stuff here...
// ================================

void BKGameScene::createScene()
{
	setSceneRect(BKConfig::getDisplayRect());
	connect(this, SIGNAL(sgnCardClicked(BKCard *)), BKController::instance(), SLOT(sltChoosenCard(BKCard *)));
}

void BKGameScene::mouseReleaseEvent(QGraphicsSceneMouseEvent * mouseEvent)
{
	QList<QGraphicsItem *> itms;

	switch(BKController::match()->matchState())
	{
		case BKMatch::MS_WAITING_USER_KICK:
			// The Game is waiting for an user input to continue..
			BKController::match()->kick();
		break;
		case BKMatch::MS_WAITING_HUM_CARD:
			// The Game is waiting for the user to choose a card
			itms = items(mouseEvent->scenePos());
			
			if (itms.count() != 0)
			{
				BKGameSceneItem * itm = dynamic_cast<BKGameSceneItem *>(itms[0]);
				if (itm && itm->type() == BKGameSceneItem::QS_ITM_CARD)
				{
					itm->clicked();
				}
			}
			
		break;
		case BKMatch::MS_STARTING_HAND:
		case BKMatch::MS_WAITING_CPU_CARD:
		case BKMatch::MS_COMPUTING_HAND:
		case BKMatch::MS_END:
		break;
	}
}

void BKGameScene::reset()
{
	QList<QGraphicsItem *> itms = m_pInstance->items();
	
	for(int i = 0; i < itms.count(); i++)
	{
		BKGameSceneItem * itm;
		if ((itm = dynamic_cast<BKGameSceneItem *>(itms[i])) != NULL)
		{
			if (itm->type() == BKGameSceneItem::QS_ITM_CARD)
			{
				itm->hide();
			}
		}
	}
}
