//=============================================================================
//
//   File : BKGameView.cpp
//   Created on mar 06 ott 2009 16:36:05 by Iacopo Palazzi
//
//   This file is part of the BrisKola distribution
//   Copyright (C) 2009 Iacopo Palazzi <iacopo.palazzi@gmail.com>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================

#include "BKGameView.h"

#include "BK.h"
#include "BKGameScene.h"
#include "BKConfig.h"
#include "BKController.h"

#include <QGraphicsPixmapItem>
#include <QGraphicsScene>
#include <QKeyEvent>
#include <QPixmap>
#include <QWidget>

// The one and only BKGameView object
BKGameView * BKGameView::m_pInstance = NULL;

BKGameView::BKGameView(QGraphicsScene * p_pScene, QWidget * p_pParent)
: QGraphicsView(p_pScene, p_pParent)
{
	createGraphics();
	connect(BKConfig::instance(), SIGNAL(sgnBackgroundChanged()), this, SLOT(setBackground()));
}

BKGameView::~BKGameView()
{
	delete m_pBackground;
	delete m_pBackgroundPixmap;
	BKGameScene::done();
}

bool BKGameView::init(QGraphicsScene * p_pScene, QWidget * p_pParent)
{
	if(m_pInstance)return false;
	m_pInstance = new BKGameView(p_pScene, p_pParent);
	return true;
}

void BKGameView::setBackground()
{
	instance()->m_pBackgroundPixmap->load(BKConfig::getImage(BKConfig::IMG_BACKGROUND));
	instance()->m_pBackground->setPixmap(* instance()->m_pBackgroundPixmap);
}

void BKGameView::done()
{
	if(!m_pInstance)return;
	delete m_pInstance;
	m_pInstance = NULL;
}


// ================================
// Start putting your stuff here...
// ================================

void BKGameView::createGraphics()
{
	m_pBackgroundPixmap = new QPixmap(BKConfig::getImage(BKConfig::IMG_BACKGROUND));
	m_pBackground = new QGraphicsPixmapItem(*m_pBackgroundPixmap);
	m_pBackground->setPos(0, 0);
	m_pBackground->setZValue(-999999);
	m_pBackground->show();
	
	BKGameScene::instance()->addItem(m_pBackground);
	
	releaseKeyboard();
	setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	setFixedSize(BKConfig::getDisplayRect().width() + 4, BKConfig::getDisplayRect().height() + 4);
}

void BKGameView::drawBackground(QPainter * painter, const QRectF & rect)
{
	//qDebug("Repaint background");
}

void BKGameView::keyPressEvent ( QKeyEvent * e )
{
	int i = -1;
	
	int key1, key2, key3, keyPress;
	
	keyPress = e->key();
	
	key1 = BKConfig::getKey("Key_0", Qt::Key_1);
	key2 = BKConfig::getKey("Key_1", Qt::Key_2);
	key3 = BKConfig::getKey("Key_2", Qt::Key_3);
	
	if(keyPress == key1)
		i = 0;
	else if(keyPress == key2)
		i = 1;
	else if(keyPress == key3)
		i = 2;
	
	if (i >= 0)
	{
		switch(BKController::match()->matchState())
		{
			case BKMatch::MS_WAITING_HUM_CARD:
				BKController::choosenCard(i);
			break;
			case BKMatch::MS_WAITING_USER_KICK:
			case BKMatch::MS_STARTING_HAND:
			case BKMatch::MS_WAITING_CPU_CARD:
			case BKMatch::MS_COMPUTING_HAND:
			case BKMatch::MS_END:
			break;
		}
	}
}
